searchData={"items":[{"type":"task","title":"mix swoosh.mailbox.server","doc":"Starts the mailbox preview server.","ref":"Mix.Tasks.Swoosh.Mailbox.Server.html"},{"type":"task","title":"Command line options - mix swoosh.mailbox.server","doc":"This task accepts the same command-line arguments as `run`.\nFor additional information, refer to the documentation for `Mix.Tasks.Run`.\n\nFor example, to run `swoosh.mailbox.server` without checking dependencies:\n\n    mix swoosh.mailbox.server --no-deps-check\n\nThe `--no-halt` flag is automatically added.","ref":"Mix.Tasks.Swoosh.Mailbox.Server.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Swoosh.Mailbox.Server.run/1","doc":"","ref":"Mix.Tasks.Swoosh.Mailbox.Server.html#run/1"},{"type":"module","title":"Swoosh","doc":"[![hex.pm](https://img.shields.io/hexpm/v/swoosh.svg)](https://hex.pm/packages/swoosh)\n[![hex.pm](https://img.shields.io/hexpm/dt/swoosh.svg)](https://hex.pm/packages/swoosh)\n[![hex.pm](https://img.shields.io/hexpm/l/swoosh.svg)](https://hex.pm/packages/swoosh)\n[![github.com](https://img.shields.io/github/last-commit/swoosh/swoosh.svg)](https://github.com/swoosh/swoosh)\n\nCompose, deliver and test your emails easily in Elixir.\n\nSwoosh comes with many adapters, including SendGrid, Mandrill, Mailgun, Postmark and SMTP.\nSee the full list of [adapters below](#adapters).\n\nThe complete documentation for Swoosh is [available online at HexDocs](https://hexdocs.pm/swoosh).","ref":"Swoosh.html"},{"type":"module","title":"Requirements - Swoosh","doc":"Elixir 1.13+ and Erlang OTP 24+","ref":"Swoosh.html#module-requirements"},{"type":"module","title":"Getting started - Swoosh","doc":"```elixir\n# In your config/config.exs file\nconfig :sample, Sample.Mailer,\n  adapter: Swoosh.Adapters.Sendgrid,\n  api_key: \"SG.x.x\"\n```\n\n```elixir\n# In your application code\ndefmodule Sample.Mailer do\n  use Swoosh.Mailer, otp_app: :sample\nend\n```\n\n```elixir\ndefmodule Sample.UserEmail do\n  import Swoosh.Email\n\n  def welcome(user) do\n    new()\n    |> to({user.name, user.email})\n    |> from({\"Dr B Banner\", \"hulk.smash@example.com\"})\n    |> subject(\"Hello, Avengers!\")\n    |> html_body(\" Hello #{user.name} \")\n    |> text_body(\"Hello #{user.name}\\n\")\n  end\nend\n```\n\n```elixir\n# In an IEx session\nemail = Sample.UserEmail.welcome(%{name: \"Tony Stark\", email: \"tony.stark@example.com\"})\nSample.Mailer.deliver(email)\n```\n\n```elixir\n# Or in a Phoenix controller\ndefmodule Sample.UserController do\n  use Phoenix.Controller\n  alias Sample.UserEmail\n  alias Sample.Mailer\n\n  def create(conn, params) do\n    user = create_user!(params)\n\n    UserEmail.welcome(user) |> Mailer.deliver()\n  end\nend\n```\n\nSee [`Swoosh.Mailer`](https://hexdocs.pm/swoosh/Swoosh.Mailer.html) for more\nconfiguration options.","ref":"Swoosh.html#module-getting-started"},{"type":"module","title":"Installation - Swoosh","doc":"- Add swoosh to your list of dependencies in `mix.exs`:\n\n  ```elixir\n  def deps do\n    [{:swoosh, \"~> 1.16\"}]\n  end\n  ```\n\n- (Optional-ish) Most adapters (non SMTP ones) use `Swoosh.ApiClient` to talk\n  to the service provider. Swoosh comes with `Swoosh.ApiClient.Hackney` configured\n  by default. If you want to use it, you just need to include\n  [`Hackney`](https://hex.pm/packages/hackney) as a dependency of your app.\n\n  Swoosh also accepts [`Finch`](https://hex.pm/packages/finch) and [`Req`](https://hex.pm/packages/req) out-of-the-box.\n  See `Swoosh.ApiClient.Finch` and `Swoosh.ApiClient.Req` for details.\n\n  If you need to integrate with another HTTP client, it's easy to define a new\n  API client. Follow the `Swoosh.ApiClient` behaviour and configure Swoosh to\n  use it:\n\n  ```elixir\n  config :swoosh, :api_client, MyApp.ApiClient\n  ```\n\n  But if you don't need `Swoosh.ApiClient`, you can disable it by setting the value\n  to `false`:\n\n  ```elixir\n  config :swoosh, :api_client, false\n  ```\n\n  This is the case when you are using `Swoosh.Adapters.Local`,\n  `Swoosh.Adapters.Test` and adapters that are SMTP based, that don't require\n  an API client.\n\n- (Optional) If you are using `Swoosh.Adapters.SMTP`,\n  `Swoosh.Adapters.Sendmail` or `Swoosh.Adapters.AmazonSES`, you also need to\n  add [`gen_smtp`](https://hex.pm/packages/gen_smtp) to your dependencies:\n\n  ```elixir\n  def deps do\n    [\n      {:swoosh, \"~> 1.6\"},\n      {:gen_smtp, \"~> 1.0\"}\n    ]\n  end\n  ```","ref":"Swoosh.html#module-installation"},{"type":"module","title":"Adapters - Swoosh","doc":"Swoosh supports the most popular transactional email providers out of the box\nand also has an SMTP adapter. Below is the list of the adapters currently\nincluded:\n\n| Provider     | Swoosh adapter                                                                                      | Remarks          |\n| ------------ | --------------------------------------------------------------------------------------------------- | ---------------- |\n| SMTP         | [Swoosh.Adapters.SMTP](https://hexdocs.pm/swoosh/Swoosh.Adapters.SMTP.html#content)                 |                  |\n| Mua          | [Swoosh.Adapters.Mua](https://hexdocs.pm/swoosh/Swoosh.Adapters.Mua.html#content)                   | SMTP alternative |\n| SendGrid     | [Swoosh.Adapters.Sendgrid](https://hexdocs.pm/swoosh/Swoosh.Adapters.Sendgrid.html#content)         |                  |\n| Brevo        | [Swoosh.Adapters.Brevo](https://hexdocs.pm/swoosh/Swoosh.Adapters.Brevo.html#content)               | Sendinblue       |\n| Sendmail     | [Swoosh.Adapters.Sendmail](https://hexdocs.pm/swoosh/Swoosh.Adapters.Sendmail.html#content)         |                  |\n| Mandrill     | [Swoosh.Adapters.Mandrill](https://hexdocs.pm/swoosh/Swoosh.Adapters.Mandrill.html#content)         |                  |\n| Mailgun      | [Swoosh.Adapters.Mailgun](https://hexdocs.pm/swoosh/Swoosh.Adapters.Mailgun.html#content)           |                  |\n| Mailjet      | [Swoosh.Adapters.Mailjet](https://hexdocs.pm/swoosh/Swoosh.Adapters.Mailjet.html#content)           |                  |\n| MsGraph      | [Swoosh.Adapters.MsGraph](https://hexdocs.pm/swoosh/Swoosh.Adapters.MsGraph.html#content)           |                  |\n| Postmark     | [Swoosh.Adapters.Postmark](https://hexdocs.pm/swoosh/Swoosh.Adapters.Postmark.html#content)         |                  |\n| SparkPost    | [Swoosh.Adapters.SparkPost](https://hexdocs.pm/swoosh/Swoosh.Adapters.SparkPost.html#content)       |                  |\n| Amazon SES   | [Swoosh.Adapters.AmazonSES](https://hexdocs.pm/swoosh/Swoosh.Adapters.AmazonSES.html#content)       |                  |\n| Amazon SES   | [Swoosh.Adapters.ExAwsAmazonSES](https://hexdocs.pm/swoosh/Swoosh.Adapters.ExAwsAmazonSES.html)     |                  |\n| Dyn          | [Swoosh.Adapters.Dyn](https://hexdocs.pm/swoosh/Swoosh.Adapters.Dyn.html#content)                   |                  |\n| Scaleway     | [Swoosh.Adapters.Scaleway](https://hexdocs.pm/swoosh/Swoosh.Adapters.Scaleway.html#content)         |                  |\n| SocketLabs   | [Swoosh.Adapters.SocketLabs](https://hexdocs.pm/swoosh/Swoosh.Adapters.SocketLabs.html#content)     |                  |\n| Gmail        | [Swoosh.Adapters.Gmail](https://hexdocs.pm/swoosh/Swoosh.Adapters.Gmail.html#content)               |                  |\n| MailPace     | [Swoosh.Adapters.MailPace](https://hexdocs.pm/swoosh/Swoosh.Adapters.MailPace.html#content)         | OhMySMTP         |\n| SMTP2GO      | [Swoosh.Adapters.SMTP2GO](https://hexdocs.pm/swoosh/Swoosh.Adapters.SMTP2GO.html#content)           |                  |\n| ProtonBridge | [Swoosh.Adapters.ProtonBridge](https://hexdocs.pm/swoosh/Swoosh.Adapters.ProtonBridge.html#content) |                  |\n| Mailtrap     | [Swoosh.Adapters.Mailtrap](https://hexdocs.pm/swoosh/Swoosh.Adapters.Mailtrap.html#content)         |                  |\n\nConfigure which adapter you want to use by updating your `config/config.exs`\nfile:\n\n```elixir\nconfig :sample, Sample.Mailer,\n  adapter: Swoosh.Adapters.SMTP\n  # adapter config (api keys, etc.)\n```\n\nCheck the documentation of the adapter you want to use for more specific\nconfigurations and instructions.\n\nAdding new adapters is super easy and we are definitely looking for\ncontributions on that front. Get in touch if you want to help!","ref":"Swoosh.html#module-adapters"},{"type":"module","title":"Recipient - Swoosh","doc":"The Recipient Protocol enables you to easily make your structs compatible\nwith Swoosh functions.\n\n```elixir\ndefmodule MyUser do\n  @derive {Swoosh.Email.Recipient, name: :name, address: :email}\n  defstruct [:name, :email, :other_props]\nend\n```\n\nNow you can directly pass `%MyUser{}` to `from`, `to`, `cc`, `bcc`, etc.\nSee `Swoosh.Email.Recipient` for more details.","ref":"Swoosh.html#module-recipient"},{"type":"module","title":"Async Emails - Swoosh","doc":"Swoosh does not make any special arrangements for sending emails in a\nnon-blocking manner. Opposite to some stacks, sending emails, talking\nto third party apps, etc in Elixir do not block or interfere with other\nrequests, so you should resort to async emails only when necessary.\n\nOne simple way to deliver emails asynchronously is by leveraging Elixir's\nstandard library. First add a Task supervisor to your application root,\nusually at `lib/my_app/application.ex`:\n\n```elixir\ndef start(_, _) do\n  children = [\n    ...,\n    # Before the endpoint\n    {Task.Supervisor, name: MyApp.AsyncEmailSupervisor},\n    MyApp.Endpoint\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one)\nend\n```\n\nNow, whenever you want to send an email:\n\n```elixir\nTask.Supervisor.start_child(MyApp.AsyncEmailSupervisor, fn ->\n  %{name: \"Tony Stark\", email: \"tony.stark@example.com\"}\n  |> Sample.UserEmail.welcome()\n  |> Sample.Mailer.deliver()\nend)\n```\n\nPlease take a look at the official docs for\n[Task](https://hexdocs.pm/elixir/Task.html) and\n[Task.Supervisor](https://hexdocs.pm/elixir/Task.Supervisor.html) for further\noptions.\n\nOne of the downsides of sending email asynchronously is that failures won't\nbe reported to the user, who won't have an opportunity to try again immediately,\nand tasks by default do not retry on errors. Therefore, if the email must be\ndelivered asynchronously, a safer solution would be to use a queue or job system.\nElixir's ecosystem has many\n[job queue libraries](https://hex.pm/packages?search=job+queue&sort=recent_downloads).\n\n- [Oban](https://hexdocs.pm/oban/Oban.html) is the current community favourite.\n  It uses PostgreSQL for storage and coordination.\n- [Exq](https://hexdocs.pm/exq/readme.html) uses Redis and is compatible with\n  Resque / Sidekiq.","ref":"Swoosh.html#module-async-emails"},{"type":"module","title":"Attachments - Swoosh","doc":"You can attach files to your email using the `Swoosh.Email.attachment/2`\nfunction. Just give the path of your file as an argument and we will do the\nrest. It also works with a `%Plug.Upload{}` struct, or a `%Swoosh.Attachment{}`\nstruct, which can be constructed using `Swoosh.Attachment.new` detailed here in\nthe [docs](https://hexdocs.pm/swoosh/Swoosh.Attachment.html#new/2).\n\nAll built-in adapters have support for attachments.\n\n```elixir\nnew()\n|> to(\"peter@example.com\")\n|> from({\"Jarvis\", \"jarvis@example.com\"})\n|> subject(\"Invoice May\")\n|> text_body(\"Here is the invoice for your superhero services in May.\")\n|> attachment(\"/Users/jarvis/invoice-peter-may.pdf\")\n```","ref":"Swoosh.html#module-attachments"},{"type":"module","title":"Testing - Swoosh","doc":"In your `config/test.exs` file set your mailer's adapter to\n`Swoosh.Adapters.Test` so that you can use the assertions provided by Swoosh in\n`Swoosh.TestAssertions` module.\n\n```elixir\ndefmodule Sample.UserTest do\n  use ExUnit.Case, async: true\n\n  import Swoosh.TestAssertions\n\n  test \"send email on user signup\" do\n    # Assuming `create_user` creates a new user then sends out a\n    # `Sample.UserEmail.welcome` email\n    user = create_user(%{username: \"ironman\", email: \"tony.stark@example.com\"})\n    assert_email_sent Sample.UserEmail.welcome(user)\n  end\nend\n```","ref":"Swoosh.html#module-testing"},{"type":"module","title":"Mailbox preview in the browser - Swoosh","doc":"Swoosh ships with a Plug that allows you to preview the emails in the local\n(in-memory) mailbox. It's particularly convenient in development when you want\nto check what your email will look like while testing the various flows of your\napplication.\n\nFor email to reach this mailbox you will need to set your `Mailer` adapter to\n`Swoosh.Adapters.Local`:\n\n```elixir\n# in config/dev.exs\nconfig :sample, MyApp.Mailer,\n  adapter: Swoosh.Adapters.Local\n```\n\nIn your Phoenix project you can `forward` directly to the plug\nwithout spinning up a separate webserver, like this:\n\n```elixir\n# in web/router.ex\nif Mix.env == :dev do\n  scope \"/dev\" do\n    pipe_through [:browser]\n\n    forward \"/mailbox\", Plug.Swoosh.MailboxPreview\n  end\nend\n```\n\nYou can also start a new server if your application does not depends on Phoenix:\n\n```elixir\n# in config/dev.exs\n# to run the preview server alongside your app\n# which may not have a web interface already\nconfig :swoosh, serve_mailbox: true\n```\n\n```elixir\n# in config/dev.exs\n# to change the preview server port (4000 by default)\nconfig :swoosh, serve_mailbox: true, preview_port: 4001\n```\n\nWhen using `serve_mailbox: true` make sure to have either `plug_cowboy` or\n`bandit` as a dependency of your app.\n\n```elixir\n{:plug_cowboy, \">= 1.0.0\"}\n# or\n{:bandit, \">= 1.0.0\"}\n```\n\nAnd finally you can also use the following Mix task to start the mailbox\npreview server independently:\n\n```console\nmix swoosh.mailbox.server\n```\n\n_Note_: the mailbox preview won't display emails\nbeing sent from outside its own node. So if you are testing using an `IEx` session,\nit's recommended to boot the application in the same session.\n`iex -S mix phx.server` or `iex -S mix swoosh.mailbox.server` will do the trick.\n\nIf you are curious, this is how it the mailbox preview looks like:\n\n![Plug.Swoosh.MailboxPreview](https://github.com/swoosh/swoosh/raw/main/images/mailbox-preview.png)\n\n\n_Note_ : To show the preview we use the cdn-version of Tailwindcss. If you have set a `content-security-policy` you may have to add `https://cdn.tailwindcss.com` to `default-src` to have the correct make up.\n\nThe preview is also available as a JSON endpoint.\n\n```sh\ncurl http://localhost:4000/dev/mailbox/json\n```","ref":"Swoosh.html#module-mailbox-preview-in-the-browser"},{"type":"module","title":"Production - Swoosh","doc":"Swoosh starts a memory storage process for local adapter by default. Normally\nit does no harm being left around in production. However, if it is causing\nproblems, or you don't like having it around, it can be disabled like so:\n\n```elixir\n# config/prod.exs\nconfig :swoosh, local: false\n```","ref":"Swoosh.html#module-production"},{"type":"module","title":"Telemetry - Swoosh","doc":"The following events are emitted:\n\n- `[:swoosh, :deliver, :start]`: occurs when `Mailer.deliver/2` begins.\n- `[:swoosh, :deliver, :stop]`: occurs when `Mailer.deliver/2` completes.\n- `[:swoosh, :deliver, :exception]`: occurs when `Mailer.deliver/2` throws an exception.\n- `[:swoosh, :deliver_many, :start]`: occurs when `Mailer.deliver_many/2` begins.\n- `[:swoosh, :deliver_many, :stop]`: occurs when `Mailer.deliver_many/2` completes.\n- `[:swoosh, :deliver_many, :exception]`: occurs when `Mailer.deliver_many/2`\n  throws an exception.\n\nView [example in docs](https://hexdocs.pm/swoosh/Swoosh.Mailer.html#module-telemetry)","ref":"Swoosh.html#module-telemetry"},{"type":"module","title":"Documentation - Swoosh","doc":"Documentation is written into the library, you will find it in the source code,\naccessible from `iex` and of course, it all gets published to\n[HexDocs](http://hexdocs.pm/swoosh).","ref":"Swoosh.html#module-documentation"},{"type":"module","title":"Contributing - Swoosh","doc":"We are grateful for any contributions. Before you submit an issue or a pull\nrequest, remember to:\n\n- Look at our [Contributing guidelines](CONTRIBUTING.md)\n- Not use the issue tracker for help or support requests (try StackOverflow,\n  IRC or Slack instead)\n- Do a quick search in the issue tracker to make sure the issues hasn't been\n  reported yet.\n- Look and follow the [Code of Conduct](CODE_OF_CONDUCT.md). Be nice and have fun!","ref":"Swoosh.html#module-contributing"},{"type":"module","title":"Running tests - Swoosh","doc":"Clone the repo and fetch its dependencies:\n\n```sh\ngit clone https://github.com/swoosh/swoosh.git\ncd swoosh\nmix deps.get\nmix test\n```","ref":"Swoosh.html#module-running-tests"},{"type":"module","title":"Building docs - Swoosh","doc":"```sh\nMIX_ENV=docs mix docs\n```","ref":"Swoosh.html#module-building-docs"},{"type":"module","title":"LICENSE - Swoosh","doc":"See [LICENSE](https://github.com/swoosh/swoosh/blob/main/LICENSE.txt)","ref":"Swoosh.html#module-license"},{"type":"behaviour","title":"Swoosh.Adapter","doc":"Specification of the email delivery adapter.","ref":"Swoosh.Adapter.html"},{"type":"callback","title":"Swoosh.Adapter.deliver/2","doc":"Delivers an email with the given config.","ref":"Swoosh.Adapter.html#c:deliver/2"},{"type":"callback","title":"Swoosh.Adapter.deliver_many/2","doc":"Delivers multiple emails with the given config in one request. Some email providers allow multiple\nmessages to be sent in one HTTP request, for example Mailjet and Postmark allow this. Check your\nprovider's documentation to see if that is possible, and see the adapter you use to find out whether\nit has been implemented.","ref":"Swoosh.Adapter.html#c:deliver_many/2"},{"type":"callback","title":"Swoosh.Adapter.validate_config/1","doc":"","ref":"Swoosh.Adapter.html#c:validate_config/1"},{"type":"function","title":"Swoosh.Adapter.validate_config/2","doc":"","ref":"Swoosh.Adapter.html#validate_config/2"},{"type":"callback","title":"Swoosh.Adapter.validate_dependency/0","doc":"","ref":"Swoosh.Adapter.html#c:validate_dependency/0"},{"type":"function","title":"Swoosh.Adapter.validate_dependency/1","doc":"","ref":"Swoosh.Adapter.html#validate_dependency/1"},{"type":"type","title":"Swoosh.Adapter.config/0","doc":"","ref":"Swoosh.Adapter.html#t:config/0"},{"type":"type","title":"Swoosh.Adapter.email/0","doc":"","ref":"Swoosh.Adapter.html#t:email/0"},{"type":"type","title":"Swoosh.Adapter.t/0","doc":"","ref":"Swoosh.Adapter.html#t:t/0"},{"type":"module","title":"Swoosh.Adapters.Local.Storage.Manager","doc":"Manages the creation/monitoring of the global in-memory storage driver,\n`Swoosh.Adapters.Local.Storage.Memory`","ref":"Swoosh.Adapters.Local.Storage.Manager.html"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Manager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Swoosh.Adapters.Local.Storage.Manager.html#child_spec/1"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Manager.start_link/1","doc":"","ref":"Swoosh.Adapters.Local.Storage.Manager.html#start_link/1"},{"type":"module","title":"Swoosh.Adapters.Local.Storage.Memory","doc":"In-memory storage driver used by the\n[Swoosh.Adapters.Local](Swoosh.Adapters.Local.html) module.\n\nThe emails in this mailbox are stored in memory and won't persist once your\napplication is stopped.","ref":"Swoosh.Adapters.Local.Storage.Memory.html"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.all/0","doc":"List all the emails in the mailbox.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#all/0"},{"type":"function","title":"Examples - Swoosh.Adapters.Local.Storage.Memory.all/0","doc":"iex> email = new |> from(\"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, [...]}\n    iex> Memory.push(email)\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}\n    iex> Memory.all()\n    [%Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}]","ref":"Swoosh.Adapters.Local.Storage.Memory.html#all/0-examples"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#child_spec/1"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.delete_all/0","doc":"Delete all the emails currently in the mailbox.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#delete_all/0"},{"type":"function","title":"Examples - Swoosh.Adapters.Local.Storage.Memory.delete_all/0","doc":"iex> email = new |> from(\"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, [...]}\n    iex> Memory.push(email)\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}\n    iex> Memory.delete_all()\n    :ok\n    iex> Memory.list()\n    []","ref":"Swoosh.Adapters.Local.Storage.Memory.html#delete_all/0-examples"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.get/1","doc":"Get a specific email from the mailbox.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#get/1"},{"type":"function","title":"Examples - Swoosh.Adapters.Local.Storage.Memory.get/1","doc":"iex> email = new |> from(\"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, [...]}\n    iex> Memory.push(email)\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}\n    iex> Memory.get(\"A1B2C3\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}","ref":"Swoosh.Adapters.Local.Storage.Memory.html#get/1-examples"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.init/1","doc":"","ref":"Swoosh.Adapters.Local.Storage.Memory.html#init/1"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.pop/0","doc":"Pop the last email from the mailbox.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#pop/0"},{"type":"function","title":"Examples - Swoosh.Adapters.Local.Storage.Memory.pop/0","doc":"iex> email = new |> from(\"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, [...]}\n    iex> Memory.push(email)\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}\n    iex> Memory.all() |> Enum.count()\n    1\n    iex> Memory.pop()\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}\n    iex> Memory.all() |> Enun.count()\n    0","ref":"Swoosh.Adapters.Local.Storage.Memory.html#pop/0-examples"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.push/1","doc":"Push a new email into the mailbox.\n\nIn order to make it easy to fetch a single email, a `Message-ID` header is\nadded to the email before being stored.","ref":"Swoosh.Adapters.Local.Storage.Memory.html#push/1"},{"type":"function","title":"Examples - Swoosh.Adapters.Local.Storage.Memory.push/1","doc":"iex> email = new |> from(\"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, [...]}\n    iex> Memory.push(email)\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, headers: %{\"Message-ID\": \"a1b2c3\"}, [...]}","ref":"Swoosh.Adapters.Local.Storage.Memory.html#push/1-examples"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.start/1","doc":"Starts the server","ref":"Swoosh.Adapters.Local.Storage.Memory.html#start/1"},{"type":"function","title":"Swoosh.Adapters.Local.Storage.Memory.stop/0","doc":"Stops the server","ref":"Swoosh.Adapters.Local.Storage.Memory.html#stop/0"},{"type":"module","title":"Swoosh.Application","doc":"","ref":"Swoosh.Application.html"},{"type":"function","title":"Swoosh.Application.start/2","doc":"","ref":"Swoosh.Application.html#start/2"},{"type":"module","title":"Swoosh.Attachment","doc":"Struct representing an attachment in an email.","ref":"Swoosh.Attachment.html"},{"type":"module","title":"Usage - Swoosh.Attachment","doc":"For all usecases of `new/2` see the function documentation.","ref":"Swoosh.Attachment.html#module-usage"},{"type":"module","title":"Inline Example - Swoosh.Attachment","doc":"new()\n    |> to({avenger.name, avenger.email})\n    |> from({\"Red Skull\", \"red_skull@villains.org\"})\n    |> subject(\"End Game invitation QR Code\")\n    |> html_body(~s| Hello #{avenger.name}  Here is your QR Code  |)\n    |> text_body(\"Hello #{avenger.name}. Please find your QR Code attached.\\n\")\n    |> attachment(\n      Swoosh.Attachment.new(\n        {:data, invitation_qr_code_binary},\n        filename: \"qrcode.png\",\n        content_type: \"image/png\",\n        type: :inline)\n    )\n    |> VillainMailer.deliver()","ref":"Swoosh.Attachment.html#module-inline-example"},{"type":"function","title":"Swoosh.Attachment.get_content/1","doc":"","ref":"Swoosh.Attachment.html#get_content/1"},{"type":"function","title":"Swoosh.Attachment.get_content/2","doc":"","ref":"Swoosh.Attachment.html#get_content/2"},{"type":"function","title":"Swoosh.Attachment.new/2","doc":"Creates a new Attachment\n\nExamples:\n\n    Attachment.new(\"/path/to/attachment.png\")\n    Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\")\n    Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\", content_type: \"image/png\")\n    Attachment.new(params[\"file\"]) # Where params[\"file\"] is a %Plug.Upload\n    Attachment.new({:data, File.read!(\"/path/to/attachment.png\")}, filename: \"image.png\", content_type: \"image/png\")\n\nExamples with inline-attachments:\n\n    Attachment.new(\"/path/to/attachment.png\", type: :inline)\n    Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\", type: :inline)\n    Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\", content_type: \"image/png\", type: :inline)\n    Attachment.new(params[\"file\"], type: :inline) # Where params[\"file\"] is a %Plug.Upload\n    Attachment.new({:data, File.read!(\"/path/to/attachment.png\")}, filename: \"image.png\", content_type: \"image/png\", type: :inline)\n\nInline attachments by default use their filename\n(or basename of the path if filename is not specified) as cid,\nin relevant adapters.\n\n    Attachment.new(\"/data/file.png\", type: :inline)\n\nGives you something like this:\n\n     \n\nYou can optionally override this default by passing in the cid option:\n\n    Attachment.new(\"/data/file.png\", type: :inline, cid: \"custom-cid\")","ref":"Swoosh.Attachment.html#new/2"},{"type":"type","title":"Swoosh.Attachment.content_encoding/0","doc":"","ref":"Swoosh.Attachment.html#t:content_encoding/0"},{"type":"type","title":"Swoosh.Attachment.t/0","doc":"","ref":"Swoosh.Attachment.html#t:t/0"},{"type":"module","title":"Swoosh.Mailer","doc":"Defines a mailer.\n\nA mailer is a wrapper around an adapter that makes it easy for you to swap the\nadapter without having to change your code.\n\nIt is also responsible for doing some sanity checks before handing down the\nemail to the adapter.\n\nWhen used, the mailer expects `:otp_app` as an option.\nThe `:otp_app` should point to an OTP application that has the mailer\nconfiguration. For example, the mailer:\n\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end\n\nCould be configured with:\n\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Sendgrid,\n      api_key: \"SG.x.x\"\n\nMost of the configuration that goes into the config is specific to the adapter,\nso check the adapter's documentation for more information.\n\nPer module configuration is also supported, it has priority over mix configs:\n\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample,\n        adapter: Swoosh.Adapters.Sendgrid,\n        api_key: \"SG.x.x\"\n    end","ref":"Swoosh.Mailer.html"},{"type":"module","title":"Usage - Swoosh.Mailer","doc":"Once configured you can use your mailer like this:\n\n    # in an IEx console\n    iex> email = new |> from(\"tony.stark@example.com\") |> to(\"steve.rogers@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, ...}\n    iex> Mailer.deliver(email)\n    {:ok, %{...}}","ref":"Swoosh.Mailer.html#module-usage"},{"type":"module","title":"Dynamic config - Swoosh.Mailer","doc":"You can also pass an extra config argument to `deliver/2` that will be merged\nwith your Mailer's config:\n\n    # in an IEx console\n    iex> email = new |> from(\"tony.stark@example.com\") |> to(\"steve.rogers@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}, ...}\n    iex> Mailer.deliver(email, domain: \"jarvis.com\")\n    {:ok, %{...}}","ref":"Swoosh.Mailer.html#module-dynamic-config"},{"type":"module","title":"Telemetry - Swoosh.Mailer","doc":"Each mailer outputs the following telemetry events:\n\n- `[:swoosh, :deliver, :start]`: occurs when `Mailer.deliver/2` begins.\n- `[:swoosh, :deliver, :stop]`: occurs when `Mailer.deliver/2` completes.\n- `[:swoosh, :deliver, :exception]`: occurs when `Mailer.deliver/2` throws an exception.\n- `[:swoosh, :deliver_many, :start]`: occurs when `Mailer.deliver_many/2` begins.\n- `[:swoosh, :deliver_many, :stop]`: occurs when `Mailer.deliver_many/2` completes.\n- `[:swoosh, :deliver_many, :exception]`: occurs when `Mailer.deliver_many/2` throws an exception.","ref":"Swoosh.Mailer.html#module-telemetry"},{"type":"module","title":"Capturing events - Swoosh.Mailer","doc":"You can capture events by calling `:telemetry.attach/4` or `:telemetry.attach_many/4`. Here's an example:\n\n    # tracks the number of emails sent successfully/errored\n    defmodule MyHandler do\n      def handle_event([:swoosh, :deliver, :stop], _measurements, metadata, _config) do\n        if Map.get(metadata, :error) do\n          StatsD.increment(\"mail.sent.failure\", 1, %{mailer: metadata.mailer})\n        else\n          StatsD.increment(\"mail.sent.success\", 1, %{mailer: metadata.mailer})\n        end\n      end\n\n      def handle_event([:swoosh, :deliver, :exception], _measurements, metadata, _config) do\n        StatsD.increment(\"mail.sent.failure\", 1, %{mailer: metadata.mailer})\n      end\n\n      def handle_event([:swoosh, :deliver_many, :stop], _measurements, metadata, _config) do\n        if Map.get(metadata, :error) do\n          StatsD.increment(\"mail.sent.failure\", length(metadata.emails), %{mailer: metadata.mailer})\n        else\n          StatsD.increment(\"mail.sent.success\", length(metadata.emails), %{mailer: metadata.mailer})\n        end\n      end\n\n      def handle_event([:swoosh, :deliver_many, :exception], _measurements, metadata, _config) do\n        StatsD.increment(\"mail.sent.failure\", length(metadata.emails), %{mailer: metadata.mailer})\n      end\n    end\n\nin `c:Application.start/2` callback:\n\n    :telemetry.attach_many(\"my-handler\", [\n       [:swoosh, :deliver, :stop],\n       [:swoosh, :deliver, :exception],\n       [:swoosh, :deliver_many, :stop],\n       [:swoosh, :deliver_many, :exception],\n     ], &MyHandler.handle_event/4, nil)","ref":"Swoosh.Mailer.html#module-capturing-events"},{"type":"function","title":"Swoosh.Mailer.deliver/2","doc":"Delivers an email.","ref":"Swoosh.Mailer.html#deliver/2"},{"type":"function","title":"Swoosh.Mailer.deliver_many/2","doc":"The implementation for `deliver_many/2` is on case-by-case basis. Check the adapter that you use\nto see if it has `deliver_many/2` implemented.","ref":"Swoosh.Mailer.html#deliver_many/2"},{"type":"module","title":"Swoosh.Email","doc":"Defines an Email.\n\nThis module defines a `Swoosh.Email` struct and the main functions for composing an email.  As it is the contract for\nthe public APIs of Swoosh it is a good idea to make use of these functions rather than build the struct yourself.","ref":"Swoosh.Email.html"},{"type":"module","title":"Email fields - Swoosh.Email","doc":"* `from` - the email address of the sender, example: `{\"Tony Stark\", \"tony.stark@example.com\"}`\n* `to` - the email address for the recipient(s), example: `[{\"Steve Rogers\", \"steve.rogers@example.com\"}]`\n* `subject` - the subject of the email, example: `\"Hello, Avengers!\"`\n* `cc` - the intended carbon copy recipient(s) of the email, example: `[{\"Bruce Banner\", \"hulk.smash@example.com\"}]`\n* `bcc` - the intended blind carbon copy recipient(s) of the email, example: `[{\"Janet Pym\", \"wasp.avengers@example.com\"}]`\n* `text_body` - the content of the email in plaintext, example: `\"Hello\"`\n* `html_body` - the content of the email in HTML, example: `\" Hello \"`\n* `reply_to` - the email address that should receive replies, example: `{\"Clint Barton\", \"hawk.eye@example.com\"}`\n* `headers` - a map of headers that should be included in the email, example: `%{\"X-Accept-Language\" => \"en-us, en\"}`\n* `attachments` - a list of attachments that should be included in the email, example: `[%{path: \"/data/uuid-random\", filename: \"att.zip\", content_type: \"application/zip\"}]`\n* `assigns` - a map of values that correspond with any template variables, example: `%{\"first_name\" => \"Bruce\"}`","ref":"Swoosh.Email.html#module-email-fields"},{"type":"module","title":"Private - Swoosh.Email","doc":"This key is reserved for use with adapters, libraries and frameworks.\n\n* `private` - a map of values that are for use by libraries/frameworks, example: `%{phoenix_template: \"welcome.html.eex\"}`\n  - `client_options` will be passed to underlining http client post call","ref":"Swoosh.Email.html#module-private"},{"type":"module","title":"Provider options - Swoosh.Email","doc":"This key allow users to make use of provider-specific functionality by passing along addition parameters.\n\n* `provider_options` - a map of values that are specific to adapter provider, example: `%{async: true}`","ref":"Swoosh.Email.html#module-provider-options"},{"type":"module","title":"Examples - Swoosh.Email","doc":"email =\n      new()\n      |> to(\"tony.stark@example.com\")\n      |> from(\"bruce.banner@example.com\")\n      |> text_body(\"Welcome to the Avengers\")\n\nThe composable nature makes it very easy to continue expanding upon a given Email.\n\n    email =\n      email\n      |> cc({\"Steve Rogers\", \"steve.rogers@example.com\"})\n      |> cc(\"wasp.avengers@example.com\")\n      |> bcc([\"thor.odinson@example.com\", {\"Henry McCoy\", \"beast.avengers@example.com\"}])\n      |> html_body(\" Special Welcome \")\n\nYou can also directly pass arguments to the `new/1` function.\n\n    email = new(from: \"tony.stark@example.com\", to: \"steve.rogers@example.com\", subject: \"Hello, Avengers!\")","ref":"Swoosh.Email.html#module-examples"},{"type":"function","title":"Swoosh.Email.assign/3","doc":"Stores a new variable key and value in the email.\n\nThis store is meant for variables used in templating. The name should be specified as an atom, the value can be any\nterm.","ref":"Swoosh.Email.html#assign/3"},{"type":"function","title":"Examples - Swoosh.Email.assign/3","doc":"iex> new() |> assign(:username, \"ironman\")\n    %Swoosh.Email{assigns: %{username: \"ironman\"}, attachments: [], bcc: [],\n     cc: [], from: nil, headers: %{}, html_body: nil, private: %{},\n     provider_options: %{}, reply_to: nil, subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#assign/3-examples"},{"type":"function","title":"Swoosh.Email.attachment/2","doc":"Add a new attachment in the email.\n\nYou can pass the path to a file, a `Swoosh.Attachment` or a `%Plug.Upload{}` struct\nas an argument. If you give a path we will detect the MIME type and determine the filename\nautomatically.\n\nYou can also send an inline-attachment used for embedding images in the body of emails by specifying `type: :inline`","ref":"Swoosh.Email.html#attachment/2"},{"type":"function","title":"Examples - Swoosh.Email.attachment/2","doc":"iex> new() |> attachment(\"/data/att.zip\")\n    %Swoosh.Email{assigns: %{}, bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: [],\n     attachments: [%Swoosh.Attachment{path: \"/data/att.zip\",\n      content_type: \"application/zip\", filename: \"att.zip\",\n      type: :attachment, data: nil, headers: [], cid: nil}]}\n    iex> new() |> attachment(Swoosh.Attachment.new(\"/data/att.zip\"))\n    %Swoosh.Email{assigns: %{}, bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: [],\n     attachments: [%Swoosh.Attachment{path: \"/data/att.zip\",\n      content_type: \"application/zip\", filename: \"att.zip\",\n      type: :attachment, data: nil, headers: [], cid: nil}]}\n    iex> new() |> attachment(%Plug.Upload{path: \"/data/abcdefg\", content_type: \"test/type\", filename: \"att.zip\"})\n    %Swoosh.Email{assigns: %{}, bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: [],\n     attachments: [%Swoosh.Attachment{path: \"/data/abcdefg\",\n      content_type: \"test/type\", filename: \"att.zip\",\n      type: :attachment, data: nil, headers: [], cid: nil}]}\n    iex> new() |> attachment(Swoosh.Attachment.new(\"/data/att.png\", type: :inline))\n    %Swoosh.Email{assigns: %{}, bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: [],\n     attachments: [%Swoosh.Attachment{path: \"/data/att.png\",\n      content_type: \"image/png\", filename: \"att.png\",\n      type: :inline, data: nil, headers: [], cid: \"att.png\"}]}","ref":"Swoosh.Email.html#attachment/2-examples"},{"type":"function","title":"Swoosh.Email.bcc/2","doc":"Adds new recipients in the `bcc` field.\n\n    iex> new() |> bcc(\"steve.rogers@example.com\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [{\"\", \"steve.rogers@example.com\"}],\n     cc: [], from: nil, headers: %{}, html_body: nil,\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"\",\n     text_body: nil, to: []}","ref":"Swoosh.Email.html#bcc/2"},{"type":"function","title":"Swoosh.Email.cc/2","doc":"Adds new recipients in the `cc` field.","ref":"Swoosh.Email.html#cc/2"},{"type":"function","title":"Examples - Swoosh.Email.cc/2","doc":"iex> new() |> cc(\"steve.rogers@example.com\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [],\n     cc: [{\"\", \"steve.rogers@example.com\"}], from: nil, headers: %{}, html_body: nil,\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"\",\n     text_body: nil, to: []}","ref":"Swoosh.Email.html#cc/2-examples"},{"type":"function","title":"Swoosh.Email.from/2","doc":"Sets a recipient in the `from` field.","ref":"Swoosh.Email.html#from/2"},{"type":"function","title":"Examples - Swoosh.Email.from/2","doc":"iex> new() |> from({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: {\"Steve Rogers\", \"steve.rogers@example.com\"},\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: []}\n\n    iex> new() |> from(\"steve.rogers@example.com\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: {\"\", \"steve.rogers@example.com\"},\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: nil, subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#from/2-examples"},{"type":"function","title":"Swoosh.Email.header/3","doc":"Adds a new `header` in the email.\n\nThe name and value must be specified as strings.","ref":"Swoosh.Email.html#header/3"},{"type":"function","title":"Examples - Swoosh.Email.header/3","doc":"iex> new() |> header(\"X-Magic-Number\", \"7\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{\"X-Magic-Number\" => \"7\"}, html_body: nil, private: %{},\n     provider_options: %{}, reply_to: nil, subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#header/3-examples"},{"type":"function","title":"Swoosh.Email.html_body/2","doc":"Sets the `html_body` field.\n\nThe HTML body must be a string that containing the HTML content.","ref":"Swoosh.Email.html#html_body/2"},{"type":"function","title":"Examples - Swoosh.Email.html_body/2","doc":"iex> new() |> html_body(\" Hello \")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [],\n     cc: [], from: nil, headers: %{}, html_body: \" Hello \",\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"\",\n     text_body: nil, to: []}","ref":"Swoosh.Email.html#html_body/2-examples"},{"type":"function","title":"Swoosh.Email.new/1","doc":"Returns a `Swoosh.Email` struct.\n\nYou can pass a keyword list or a map argument to the function that will be used\nto populate the fields of that struct. Note that it will silently ignore any\nfields that it doesn't know about.","ref":"Swoosh.Email.html#new/1"},{"type":"function","title":"Examples - Swoosh.Email.new/1","doc":"iex> new()\n    %Swoosh.Email{}\n\n    iex> new(subject: \"Hello, Avengers!\")\n    %Swoosh.Email{subject: \"Hello, Avengers!\"}\n\n    iex> new(from: \"tony.stark@example.com\")\n    %Swoosh.Email{from: {\"\", \"tony.stark@example.com\"}}\n    iex> new(from: {\"Tony Stark\", \"tony.stark@example.com\"})\n    %Swoosh.Email{from: {\"Tony Stark\", \"tony.stark@example.com\"}}\n\n    iex> new(to: \"steve.rogers@example.com\")\n    %Swoosh.Email{to: [{\"\", \"steve.rogers@example.com\"}]}\n    iex> new(to: {\"Steve Rogers\", \"steve.rogers@example.com\"})\n    %Swoosh.Email{to: [{\"Steve Rogers\", \"steve.rogers@example.com\"}]}\n    iex> new(to: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, \"thor.odinson@example.com\"])\n    %Swoosh.Email{to: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, {\"\", \"thor.odinson@example.com\"}]}\n\n    iex> new(cc: \"steve.rogers@example.com\")\n    %Swoosh.Email{cc: [{\"\", \"steve.rogers@example.com\"}]}\n    iex> new(cc: {\"Steve Rogers\", \"steve.rogers@example.com\"})\n    %Swoosh.Email{cc: [{\"Steve Rogers\", \"steve.rogers@example.com\"}]}\n    iex> new(cc: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, \"thor.odinson@example.com\"])\n    %Swoosh.Email{cc: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, {\"\", \"thor.odinson@example.com\"}]}\n\n    iex> new(bcc: \"steve.rogers@example.com\")\n    %Swoosh.Email{bcc: [{\"\", \"steve.rogers@example.com\"}]}\n    iex> new(bcc: {\"Steve Rogers\", \"steve.rogers@example.com\"})\n    %Swoosh.Email{bcc: [{\"Steve Rogers\", \"steve.rogers@example.com\"}]}\n    iex> new(bcc: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, \"thor.odinson@example.com\"])\n    %Swoosh.Email{bcc: [{\"Bruce Banner\", \"bruce.banner@example.com\"}, {\"\", \"thor.odinson@example.com\"}]}\n\n    iex> new(html_body: \" Welcome, Avengers \")\n    %Swoosh.Email{html_body: \" Welcome, Avengers \"}\n\n    iex> new(text_body: \"Welcome, Avengers\")\n    %Swoosh.Email{text_body: \"Welcome, Avengers\"}\n\n    iex> new(reply_to: \"edwin.jarvis@example.com\")\n    %Swoosh.Email{reply_to: {\"\", \"edwin.jarvis@example.com\"}}\n    iex> new(reply_to: {\"Edwin Jarvis\", \"edwin.jarvis@example.com\"})\n    %Swoosh.Email{reply_to: {\"Edwin Jarvis\", \"edwin.jarvis@example.com\"}}\n\n    iex> new(headers: %{\"X-Accept-Language\" => \"en\"})\n    %Swoosh.Email{headers: %{\"X-Accept-Language\" => \"en\"}}\n\n    iex> new(assigns: %{user_id: 10})\n    %Swoosh.Email{assigns: %{user_id: 10}}\n\n    iex> new(provider_options: %{async: true})\n    %Swoosh.Email{provider_options: %{async: true}}\n\nYou can obviously combine these arguments together:\n\n    iex> new(to: \"steve.rogers@example.com\", subject: \"Hello, Avengers!\")\n    %Swoosh.Email{to: [{\"\", \"steve.rogers@example.com\"}], subject: \"Hello, Avengers!\"}","ref":"Swoosh.Email.html#new/1-examples"},{"type":"function","title":"Swoosh.Email.put_bcc/2","doc":"Puts new recipients in the `bcc` field.\n\nIt will replace any previously added `bcc` recipients.","ref":"Swoosh.Email.html#put_bcc/2"},{"type":"function","title":"Swoosh.Email.put_cc/2","doc":"Puts new recipients in the `cc` field.\n\nIt will replace any previously added `cc` recipients.","ref":"Swoosh.Email.html#put_cc/2"},{"type":"function","title":"Swoosh.Email.put_private/3","doc":"Stores a new **private** key and value in the email.\n\nThis store is meant to be for libraries/framework usage. The name should be\nspecified as an atom, the value can be any term.","ref":"Swoosh.Email.html#put_private/3"},{"type":"function","title":"Examples - Swoosh.Email.put_private/3","doc":"iex> new() |> put_private(:phoenix_template, \"welcome.html\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{phoenix_template: \"welcome.html\"},\n     provider_options: %{}, reply_to: nil, subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#put_private/3-examples"},{"type":"function","title":"Swoosh.Email.put_provider_option/3","doc":"Stores a new **provider_option** key and value in the email.\n\nThis store is meant for adapter usage, to aid provider-specific functionality.\nThe name should be specified as an atom, the value can be any term.","ref":"Swoosh.Email.html#put_provider_option/3"},{"type":"function","title":"Examples - Swoosh.Email.put_provider_option/3","doc":"iex> new() |> put_provider_option(:async, true)\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{async: true},\n     reply_to: nil, subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#put_provider_option/3-examples"},{"type":"function","title":"Swoosh.Email.put_to/2","doc":"Puts new recipients in the `to` field.\n\nIt will replace any previously added `to` recipients.","ref":"Swoosh.Email.html#put_to/2"},{"type":"function","title":"Swoosh.Email.reply_to/2","doc":"Sets a recipient in the `reply_to` field. May also set a list of recipients as `reply_to`, but the\nsupport for it on adapters is on case-by-case basis.","ref":"Swoosh.Email.html#reply_to/2"},{"type":"function","title":"Examples - Swoosh.Email.reply_to/2","doc":"iex> new() |> reply_to({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: {\"Steve Rogers\", \"steve.rogers@example.com\"}, subject: \"\", text_body: nil, to: []}\n\n    iex> new() |> reply_to(\"steve.rogers@example.com\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: {\"\", \"steve.rogers@example.com\"}, subject: \"\", text_body: nil, to: []}\n\n    iex> new() |> reply_to([{\"Steve Rogers\", \"steve.rogers@example.com\"}, \"bucky.barnes@example.com\"])\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [], cc: [], from: nil,\n     headers: %{}, html_body: nil, private: %{}, provider_options: %{},\n     reply_to: [{\"Steve Rogers\", \"steve.rogers@example.com\"}, {\"\", \"bucky.barnes@example.com\"}],\n     subject: \"\", text_body: nil, to: []}","ref":"Swoosh.Email.html#reply_to/2-examples"},{"type":"function","title":"Swoosh.Email.subject/2","doc":"Sets the `subject` field.\n\nThe subject must be a string that contains the subject.","ref":"Swoosh.Email.html#subject/2"},{"type":"function","title":"Examples - Swoosh.Email.subject/2","doc":"iex> new() |> subject(\"Hello, Avengers!\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [],\n     cc: [], from: nil, headers: %{}, html_body: nil,\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"Hello, Avengers!\",\n     text_body: nil, to: []}","ref":"Swoosh.Email.html#subject/2-examples"},{"type":"function","title":"Swoosh.Email.text_body/2","doc":"Sets the `text_body` field.\n\nThe text body must be a string that containing the plaintext content.","ref":"Swoosh.Email.html#text_body/2"},{"type":"function","title":"Examples - Swoosh.Email.text_body/2","doc":"iex> new() |> text_body(\"Hello\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [],\n     cc: [], from: nil, headers: %{}, html_body: nil,\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"\",\n     text_body: \"Hello\", to: []}","ref":"Swoosh.Email.html#text_body/2-examples"},{"type":"function","title":"Swoosh.Email.to/2","doc":"Adds new recipients in the `to` field.","ref":"Swoosh.Email.html#to/2"},{"type":"function","title":"Examples - Swoosh.Email.to/2","doc":"iex> new() |> to(\"steve.rogers@example.com\")\n    %Swoosh.Email{assigns: %{}, attachments: [], bcc: [],\n     cc: [], from: nil, headers: %{}, html_body: nil,\n     private: %{}, provider_options: %{}, reply_to: nil, subject: \"\",\n     text_body: nil, to: [{\"\", \"steve.rogers@example.com\"}]}","ref":"Swoosh.Email.html#to/2-examples"},{"type":"type","title":"Swoosh.Email.address/0","doc":"","ref":"Swoosh.Email.html#t:address/0"},{"type":"type","title":"Swoosh.Email.html_body/0","doc":"","ref":"Swoosh.Email.html#t:html_body/0"},{"type":"type","title":"Swoosh.Email.mailbox/0","doc":"","ref":"Swoosh.Email.html#t:mailbox/0"},{"type":"type","title":"Swoosh.Email.name/0","doc":"","ref":"Swoosh.Email.html#t:name/0"},{"type":"type","title":"Swoosh.Email.subject/0","doc":"","ref":"Swoosh.Email.html#t:subject/0"},{"type":"type","title":"Swoosh.Email.t/0","doc":"","ref":"Swoosh.Email.html#t:t/0"},{"type":"type","title":"Swoosh.Email.text_body/0","doc":"","ref":"Swoosh.Email.html#t:text_body/0"},{"type":"protocol","title":"Swoosh.Email.Recipient","doc":"Recipient Protocol controls how data is formatted into an email recipient","ref":"Swoosh.Email.Recipient.html"},{"type":"protocol","title":"Deriving - Swoosh.Email.Recipient","doc":"The protocol allows leveraging the Elixir's `@derive` feature to simplify protocol implementation\nin trivial cases. Accepted options are:\n\n  * `:name` (optional)\n  * `:address` (required)","ref":"Swoosh.Email.Recipient.html#module-deriving"},{"type":"protocol","title":"Example - Swoosh.Email.Recipient","doc":"defmodule MyUser do\n      @derive {Swoosh.Email.Recipient, name: :name, address: :email}\n      defstruct [:name, :email, :other_props]\n    end\n\nor with optional name...\n\n    defmodule MySubscriber do\n      @derive {Swoosh.Email.Recipient, address: :email}\n      defstruct [:email, :preferences]\n    end\n\nfull implementation without deriving...\n\n    defmodule MyUser do\n      defstruct [:name, :email, :other_props]\n    end\n\n    defimpl Swoosh.Email.Recipient, for: MyUser do\n      def format(%MyUser{name: name, email: address} = value) do\n        {name, address}\n      end\n    end","ref":"Swoosh.Email.Recipient.html#module-example"},{"type":"function","title":"Swoosh.Email.Recipient.format/1","doc":"Formats `value` into a Swoosh recipient, a 2-tuple with recipient name and recipient address","ref":"Swoosh.Email.Recipient.html#format/1"},{"type":"type","title":"Swoosh.Email.Recipient.t/0","doc":"","ref":"Swoosh.Email.Recipient.html#t:t/0"},{"type":"module","title":"Swoosh.Adapters.AmazonSES","doc":"An adapter that sends email using the Amazon Simple Email Service (SES) Query API.\nThis adapter does not depend on `ExAws`; if you are already using it, you may\nprefer `Swoosh.Adapters.ExAwsAmazonSES`.\n\nThis email adapter makes use of the Amazon SES SendRawEmail action and generates\na SMTP style message containing the information to be emailed. This allows for\ngreater more customizable email message and ensures the capability to add\nattachments. As a result, however, the `:gen_smtp` dependency is required in order\nto correctly generate the SMTP message that will be sent.\n\nEnsure sure you have the dependency added in your mix.exs file:\n\n    def deps do\n      [\n        {:swoosh, \"~> 1.0\"},\n        {:gen_smtp, \"~> 1.0\"}\n      ]\n    end\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.\n\nSee also:\n\n[Amazon SES Query Api Docs](http://docs.aws.amazon.com/ses/latest/APIReference/Welcome.html)\n\n[Amazon SES SendRawEmail Documentation](http://docs.aws.amazon.com/ses/latest/APIReference/API_SendRawEmail.html)","ref":"Swoosh.Adapters.AmazonSES.html"},{"type":"module","title":"Configuration options - Swoosh.Adapters.AmazonSES","doc":"","ref":"Swoosh.Adapters.AmazonSES.html#module-configuration-options"},{"type":"module","title":"Required - Swoosh.Adapters.AmazonSES","doc":"Note that these are handled automatically if using `Swoosh.Adapters.ExAwsAmazonSES`.\n\n* `:region` - the AWS region\n* `:access_key` - the IAM access key\n* `:secret` - the IAM secret","ref":"Swoosh.Adapters.AmazonSES.html#module-required"},{"type":"module","title":"Optional - Swoosh.Adapters.AmazonSES","doc":"* `:ses_source` - An email address used for bounce reports. Will be set as the\n  `Source` [request parameter](https://docs.aws.amazon.com/ses/latest/APIReference/API_SendRawEmail.html#API_SendRawEmail_RequestParameters)\n   in the SES API. Note that this must correspond to a verified identity.","ref":"Swoosh.Adapters.AmazonSES.html#module-optional"},{"type":"module","title":"Examples - Swoosh.Adapters.AmazonSES","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.AmazonSES,\n      region: \"region-endpoint\",\n      access_key: \"aws-access-key\",\n      secret: \"aws-secret-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.AmazonSES.html#module-examples"},{"type":"module","title":"Using with tags and configuration set - Swoosh.Adapters.AmazonSES","doc":"import Swoosh.Email\n\n    new()\n    |> from(\"guybrush.threepwood@pirates.grog\")\n    |> to(\"elaine.marley@triisland.gov\")\n    |> subject(\"Mighty Pirate Newsletter\")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:tags, [%{name: \"name1\", value: \"test1\"}])\n    |> put_provider_option(:configuration_set_name, \"configuration_set_name1\")","ref":"Swoosh.Adapters.AmazonSES.html#module-using-with-tags-and-configuration-set"},{"type":"module","title":"Provider Options - Swoosh.Adapters.AmazonSES","doc":"* `:tags` (list[map]) - a list of key/value pairs of a tag\n  * `:configuration_set_name` (string) - the name of the configuration set\n  * `:security_token` (string) - temporary security token obtained through\n    AWS Security Token Service (AWS STS)","ref":"Swoosh.Adapters.AmazonSES.html#module-provider-options"},{"type":"module","title":"IAM role - Swoosh.Adapters.AmazonSES","doc":"In case you use IAM role for authenticating AWS requests, you can fetch\ntemporary `:access_key` and `:secret_key` from that role, but you also need to\ninclude additional `X-Amz-Security-Token` header to that request.\n\nYou can do that by adding `:security_token` to `:provider_options`.\n\nIf you don't have a static `:access_key` and `:secret_key` for your\napplication, you can use the `Swoosh.Adapters.ExAwsAmazonSES` adapter to fetch credentials\non-demand as specified in your application's `:ex_aws` configuration.","ref":"Swoosh.Adapters.AmazonSES.html#module-iam-role"},{"type":"function","title":"Swoosh.Adapters.AmazonSES.deliver/2","doc":"","ref":"Swoosh.Adapters.AmazonSES.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.AmazonSES.validate_config/1","doc":"","ref":"Swoosh.Adapters.AmazonSES.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.AmazonSES.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.AmazonSES.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Brevo","doc":"An adapter that sends email using the Brevo API (Transactional emails only).\n\nFor reference: [Brevo API docs](https://developers.brevo.com/reference/sendtransacemail)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Brevo.html"},{"type":"module","title":"Example - Swoosh.Adapters.Brevo","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Brevo,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Brevo.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Brevo","doc":"import Swoosh.Email\n\n    new()\n    |> from(\"nora@example.com\")\n    |> to(\"shushu@example.com\")\n    |> subject(\"Hello, Wally!\")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:id, 42)\n    |> put_provider_option(:template_id, 42)\n    |> put_provider_option(:params, %{param1: \"a\", param2: 123})\n    |> put_provider_option(:tags, [\"tag_1\", \"tag_2\"])\n    |> put_provider_option(:schedule_at, ~U[2022-11-15 11:00:00Z])","ref":"Swoosh.Adapters.Brevo.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Brevo","doc":"* `sender_id` (integer) - `sender`, the sender `id` where this library will\n    add email obtained from the `from/1`\n\n  * `template_id` (integer) - `templateId`, the Id of the `active`\n    transactional email template\n\n  * `params` (map) - `params`, a map of key/value attributes to customize the\n    template\n\n  * `tags` (list[string]) - `tags`, a list of tags for each email for easy\n    filtering\n\n  * `schedule_at` (UTC DateTime) - `schedule_at`, a UTC date-time on which the email has to schedule","ref":"Swoosh.Adapters.Brevo.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Brevo.deliver/2","doc":"","ref":"Swoosh.Adapters.Brevo.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Brevo.validate_config/1","doc":"","ref":"Swoosh.Adapters.Brevo.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Brevo.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Brevo.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.CustomerIO","doc":"An adapter that sends email using the CustomerIO API.\n\nFor reference: [CustomerIO API docs](https://customer.io/docs/api/#tag/Transactional)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.CustomerIO.html"},{"type":"module","title":"Example - Swoosh.Adapters.CustomerIO","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.CustomerIO,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.CustomerIO.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.CustomerIO","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Xu Shang-Chi\", \"xu.shangchi@example.com\"})\n    |> to({\"Katy\", \"katy@example.com\"})\n    |> reply_to(\"xu.xialing@example.com\")\n    |> cc(\"yingli@example.com\")\n    |> cc({\"Xu Wenwu\", \"xu.wenwu@example.com\"})\n    |> bcc(\"yingnan@example.com\")\n    |> bcc({\"Jon Jon\", \"jonjon@example.com\"})\n    |> subject(\"Hello, Ten Rings!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:disable_css_preprocessing, true)\n    |> put_provider_option(:disable_message_retention, true)\n    |> put_provider_option(:fake_bcc, true)\n    |> put_provider_option(:message_data, %{\n      my_var: %{my_message_id: 123},\n      my_other_var: %{my_other_id: 1, stuff: 2}\n    })\n    |> put_provider_option(:preheader, \"this is the preview\")\n    |> put_provider_option(:queue_draft, true)\n    |> put_provider_option(:send_at, 1617260400)\n    |> put_provider_option(:send_to_unsubscribed, true)\n    |> put_provider_option(:tracked, false)\n    |> put_provider_option(:transactional_message_id, 44)","ref":"Swoosh.Adapters.CustomerIO.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.CustomerIO","doc":"Supported provider options are the following:\n\n#### Inserted into request body\n\n  * `:disable_css_preprocessing` (boolean) - Set to true to disable CSS preprocessing.\n     This setting overrides the CSS preprocessing setting on the transactional_message_id\n     as set in the user interface. Transactional emails have CSS preprocessing enabled by\n     default.\n\n  * `:disable_message_retention` (boolean) - If true, the message body is not\n    retained in delivery history. Setting this value overrides the value set\n    in the settings of your transactional_message_id.\n\n  * `:identifiers` (map) - Identifies the person represented by your transactional\n    message by one of, and only one of, id, email, or cio_id.\n\n  * `:fake_bcc` (boolean) - If true, rather than sending true copies to BCC\n    addresses, Customer.io sends a copy of the message with the subject line\n    containing the recipient address(es).\n\n  * `:message_data` (map) - An object containing the key-value pairs referenced\n      using liquid in your message, see `:transactional_message_id`.\n\n  * `:preheader` (string) - Also known as \"preview text\", this is the block block\n    of text that users see next to, or underneath, the subject line in their inbox.\n\n  * `:queue_draft` (boolean) - If true, your transactional message is held as\n    a draft in Customer.io and not sent directly to your audience. You must go\n    to the Deliveries and Drafts page to send your message.\n\n  * `:send_at` (integer) - A unix timestamp determining when the message will be sent.\n    The timestamp can be up to 90 days in the future. If this value is in the past,\n    your message is sent immediately.\n\n  * `:send_to_unsubscribed` (boolean) - If false, your message is not sent to\n    unsubscribed recipients. Setting this value overrides the value set in the\n    settings of your transactional_message_id.\n\n  * `:tracked\"` (boolean) - If true, Customer.io tracks opens and link clicks\n    in your message.\n\n  * `:transactional_message_id` (integer or string) - The transactional message template that\n    you want to use for your message. You can call the template by its numerical ID\n    or by the Trigger Name that you assigned the template (case insensitive).","ref":"Swoosh.Adapters.CustomerIO.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.CustomerIO.validate_config/1","doc":"","ref":"Swoosh.Adapters.CustomerIO.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.CustomerIO.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.CustomerIO.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Dyn","doc":"An adapter that sends email using the Dyn API.\n\nFor reference: [Dyn API docs](https://help.dyn.com/email-rest-methods-api/sending-api/)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Dyn.html"},{"type":"module","title":"Example - Swoosh.Adapters.Dyn","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Dyn,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Dyn.html#module-example"},{"type":"module","title":"Sending sample email - Swoosh.Adapters.Dyn","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Christine\", \"christine@example.com\"})\n    |> to({\"constance\", \"constance@example.com\"})\n    |> to(\"ming_fleetfoot@example.com\")\n    |> bcc([\n      {\"Dr. Xander Bravestone\", \"dr.xander_bravestone@example.com\"},\n      {\"Prof. Sheldon Oberon\", \"prof.sheldon.oberon@example.com\"}\n    ])\n    |> subject(\"Hello, People!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")","ref":"Swoosh.Adapters.Dyn.html#module-sending-sample-email"},{"type":"function","title":"Swoosh.Adapters.Dyn.deliver/2","doc":"","ref":"Swoosh.Adapters.Dyn.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Dyn.validate_config/1","doc":"","ref":"Swoosh.Adapters.Dyn.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Dyn.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Dyn.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.ExAwsAmazonSES","doc":"An adapter that wraps `Swoosh.Adapters.AmazonSES` to use credentials from `ExAws`.\n\nYou may prefer this adapter to `Swoosh.Adapters.AmazonSES` if you have already\nconfigured ExAws for your project and are using instance role credentials.\n\nThis allows you to use automatically managed, short-lived credentials, rather than provisioning\na static access key / secret key pair.\n\nSee also:\n\n[IAM Roles for EC2](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html)\n[IAM roles for ECS tasks](https://docs.aws.amazon.com/AmazonECS/latest/userguide/task-iam-roles.html)\n\n>","ref":"Swoosh.Adapters.ExAwsAmazonSES.html"},{"type":"module","title":"Dependencies {: .info} - Swoosh.Adapters.ExAwsAmazonSES","doc":">\n> In addition to the `:gen_smtp` dependency that the `AmazonSES` adapter\n> requires, this adapter also depends on `:ex_aws`.\n>\n> Ensure you have the dependencies added in your mix.exs file:\n>\n>     def deps do\n>       [\n>         {:swoosh, \"~> 1.0\"},\n>         {:gen_smtp, \"~> 1.0\"},\n>         {:ex_aws, \"~> 2.1\"},\n>         # Dependency of `:ex_aws`\n>         {:sweet_xml, \"~> 0.6\"}\n>       ]\n>     end","ref":"Swoosh.Adapters.ExAwsAmazonSES.html#module-dependencies-info"},{"type":"module","title":"Example - Swoosh.Adapters.ExAwsAmazonSES","doc":"# config/config.exs\n    config :ex_aws,\n      access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, :instance_role],\n      secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, :instance_role],\n      region: \"us-east-1\"\n\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.ExAwsAmazonSES\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end\n\nSee also:\n\n[Getting started with ExAws](https://hexdocs.pm/ex_aws/readme.html#getting-started)","ref":"Swoosh.Adapters.ExAwsAmazonSES.html#module-example"},{"type":"function","title":"Swoosh.Adapters.ExAwsAmazonSES.deliver/2","doc":"","ref":"Swoosh.Adapters.ExAwsAmazonSES.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.ExAwsAmazonSES.validate_config/1","doc":"","ref":"Swoosh.Adapters.ExAwsAmazonSES.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.ExAwsAmazonSES.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.ExAwsAmazonSES.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Gmail","doc":"An adapter that sends email using Gmail api\n\nFor reference: [Gmail API docs](https://developers.google.com/gmail/api)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.\n\n>","ref":"Swoosh.Adapters.Gmail.html"},{"type":"module","title":"Dependency {: .info} - Swoosh.Adapters.Gmail","doc":">\n> Gmail adapter requires `Mail` dependency to format message as RFC 2822 message.\n>\n>     {:mail, \">= 0.0.0\"}\n\nBecause `Mail` library removes Bcc headers, they are being added after email is\nrendered, in adapter code.","ref":"Swoosh.Adapters.Gmail.html#module-dependency-info"},{"type":"module","title":"Example - Swoosh.Adapters.Gmail","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Gmail,\n      access_token: {:system, \"GMAIL_API_ACCESS_TOKEN\"}\n\n    # To deal with token refresh, it could be a better idea to pass the access token\n    # in via deliver config explicitly, if you don't update the environment variable\n    # periodically. e.g.\n    MyMailer.deliver(my_email, access_token: my_access_token)\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Gmail.html#module-example"},{"type":"module","title":"Required config parameters - Swoosh.Adapters.Gmail","doc":"- `:access_token` valid OAuth2 access token\n      Required scopes:\n      - gmail.compose\n    See https://developers.google.com/oauthplayground when developing","ref":"Swoosh.Adapters.Gmail.html#module-required-config-parameters"},{"type":"function","title":"Swoosh.Adapters.Gmail.deliver/2","doc":"","ref":"Swoosh.Adapters.Gmail.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Gmail.validate_config/1","doc":"","ref":"Swoosh.Adapters.Gmail.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Gmail.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Gmail.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Local","doc":"An adapter that stores the email locally, using the specified storage driver.\n\nThis is especially useful in development to avoid sending real emails. You can\nread the emails you have sent using functions in the\n[Swoosh.Adapters.Local.Storage.Memory](Swoosh.Adapters.Local.Storage.Memory.html)\nor the [Plug.Swoosh.MailboxPreview](Plug.Swoosh.MailboxPreview.html) plug.","ref":"Swoosh.Adapters.Local.html"},{"type":"module","title":"Example - Swoosh.Adapters.Local","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Local\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Local.html#module-example"},{"type":"function","title":"Swoosh.Adapters.Local.deliver/2","doc":"","ref":"Swoosh.Adapters.Local.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Local.deliver_many/2","doc":"","ref":"Swoosh.Adapters.Local.html#deliver_many/2"},{"type":"function","title":"Swoosh.Adapters.Local.validate_config/1","doc":"","ref":"Swoosh.Adapters.Local.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Local.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Local.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Logger","doc":"An adapter that only logs email using Logger.\n\nIt can be useful in environment where you do not necessarily want to send real\nemails (eg. staging environments) or in development.\n\nBy default it only prints the recipient of the email but you can print the full\nemail by using `log_full_email: true` in the adapter configuration.","ref":"Swoosh.Adapters.Logger.html"},{"type":"module","title":"Example - Swoosh.Adapters.Logger","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Logger,\n      level: :debug\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Logger.html#module-example"},{"type":"function","title":"Swoosh.Adapters.Logger.deliver/2","doc":"","ref":"Swoosh.Adapters.Logger.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Logger.validate_config/1","doc":"","ref":"Swoosh.Adapters.Logger.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Logger.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Logger.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.MailPace","doc":"An adapter that sends email using the MailPace API.\n\nFor reference: [MailPace API docs](https://docs.mailpace.com/reference/overview/)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.MailPace.html"},{"type":"module","title":"Example - Swoosh.Adapters.MailPace","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.MailPace,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.MailPace.html#module-example"},{"type":"function","title":"Swoosh.Adapters.MailPace.deliver/2","doc":"","ref":"Swoosh.Adapters.MailPace.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.MailPace.validate_config/1","doc":"","ref":"Swoosh.Adapters.MailPace.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.MailPace.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.MailPace.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Mailgun","doc":"An adapter that sends email using the Mailgun API.\n\nFor reference: [Mailgun API docs](https://documentation.mailgun.com/en/latest/api-sending.html#sending)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.\n\n>","ref":"Swoosh.Adapters.Mailgun.html"},{"type":"module","title":"Dependency {: .info} - Swoosh.Adapters.Mailgun","doc":">\n> Mailgun adapter requires `Plug` and [`:multipart`](https://hex.pm/packages/multipart) to work properly.","ref":"Swoosh.Adapters.Mailgun.html#module-dependency-info"},{"type":"module","title":"Configuration options - Swoosh.Adapters.Mailgun","doc":"* `:api_key` - the API key used with Mailgun\n* `:domain` - the domain you will be sending emails from. For sandbox domains, make sure to use the sandbox address, for example: `https://api.mailgun.net/v3/sandbox123456.mailgun.org/messages` then you should set `domain: \"sandbox123456.mailgun.org\"`.\n* `:base_url` - the url to use as the API endpoint. For EU domains, use https://api.eu.mailgun.net/v3","ref":"Swoosh.Adapters.Mailgun.html#module-configuration-options"},{"type":"module","title":"Example - Swoosh.Adapters.Mailgun","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mailgun,\n      api_key: \"my-api-key\",\n      domain: \"avengers.com\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Mailgun.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Mailgun","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"T Stark\", \"tony.stark@example.com\"})\n    |> to({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    |> to(\"wasp.avengers@example.com\")\n    |> reply_to(\"office.avengers@example.com\")\n    |> cc({\"Bruce Banner\", \"hulk.smash@example.com\"})\n    |> cc(\"thor.odinson@example.com\")\n    |> bcc({\"Clinton Francis Barton\", \"hawk.eye@example.com\"})\n    |> bcc(\"beast.avengers@example.com\")\n    |> subject(\"Hello, Avengers!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:custom_vars, %{\"key\" => \"value\"})\n    |> put_provider_option(:recipient_vars, %{\"steve.rogers@example.com\": %{var1: 123}, \"juan.diaz@example.com\": %{var1: 456}})\n    |> put_provider_option(:sending_options, %{dkim: \"yes\", tracking: \"no\"})\n    |> put_provider_option(:tags, [\"worldwide-peace\", \"unity\"])\n    |> put_provider_option(:template_name, \"avengers-templates\")\n    |> put_provider_option(:template_options, %{version: \"initial\"})","ref":"Swoosh.Adapters.Mailgun.html#module-using-with-provider-options"},{"type":"module","title":"Provider options - Swoosh.Adapters.Mailgun","doc":"* `:custom_vars` (map) - used to translate to `v:my-var`, now\n    `h:X-Mailgun-Variables`, add custom data to email\n\n  * `:recipient_vars` (map) - `recipient-variables`, assign\n    custom variable for each email recipient\n\n  * `:sending_options` (map) - `o:my-key`, all the sending options\n\n  * `:tags` (list[string]) - `o:tag`, was added in before `:sending_options`,\n    kept for backward compatibility, use `:sending_options` instead\n\n  * `:template_name` (string) - `template`, name of template created at Mailgun\n\n  * `:template_options` (map) - `version`, `text`, `variables` and/or any future possible values","ref":"Swoosh.Adapters.Mailgun.html#module-provider-options"},{"type":"module","title":"Custom headers - Swoosh.Adapters.Mailgun","doc":"Headers added via `Email.header/3` will be translated to (`h:`) values that\nMailgun recognizes.","ref":"Swoosh.Adapters.Mailgun.html#module-custom-headers"},{"type":"function","title":"Swoosh.Adapters.Mailgun.deliver/2","doc":"","ref":"Swoosh.Adapters.Mailgun.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Mailgun.validate_config/1","doc":"","ref":"Swoosh.Adapters.Mailgun.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Mailgun.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Mailgun.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Mailjet","doc":"An adapter that sends email using the Mailjet API.\n\nFor reference: [Mailjet API docs](https://dev.mailjet.com/guides/#send-api-v3-1)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.\n\n>","ref":"Swoosh.Adapters.Mailjet.html"},{"type":"module","title":"Dependency {: .info} - Swoosh.Adapters.Mailjet","doc":">\n> Mailjet adapter requires `Plug` to work properly.","ref":"Swoosh.Adapters.Mailjet.html#module-dependency-info"},{"type":"module","title":"Example - Swoosh.Adapters.Mailjet","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mailjet,\n      api_key: \"my-api-key\",\n      secret: \"my-secret-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Mailjet.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Mailjet","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Billi Wang\", \"billi_wang@example.com\"})\n    |> to({\"Nai Nai\", \"nainai@example.com\"})\n    |> reply_to(\"a24@example.com\")\n    |> cc({\"Haiyan Wang\", \"haiyan_wang@example.com\"})\n    |> cc(\"lujian@example.com\")\n    |> bcc({\"Hao Hao\", \"haohao@example.com\"})\n    |> bcc(\"aiko@example.com\")\n    |> subject(\"Hello, Nai Nai!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:template_id, 123)\n    |> put_provider_option(:template_error_deliver, true)\n    |> put_provider_option(:template_error_reporting, \"developer@example.com\")\n    |> put_provider_option(:variables, %{firstname: \"lulu\", lastname: \"wang\"})","ref":"Swoosh.Adapters.Mailjet.html#module-using-with-provider-options"},{"type":"module","title":"Provider options - Swoosh.Adapters.Mailjet","doc":"* `:template_id` (integer) - `TemplateID`, unique template id of the\n    template to be used as email content\n\n  * `:template_error_deliver` (boolean) - `TemplateErrorDeliver`,\n    send even if error in template if `true`, otherwise stop email delivery\n    immediately upon error\n\n  * `:template_error_reporting` (string | tuple | map) - `TemplateErrorReporting`,\n    email address or a tuple of name and email address of a recipient to send a\n    carbon copy upon error\n\n  * `:variables` (map) - `Variables`, custom key-value variable for the email\n    content","ref":"Swoosh.Adapters.Mailjet.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Mailjet.deliver/2","doc":"","ref":"Swoosh.Adapters.Mailjet.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Mailjet.deliver_many/2","doc":"","ref":"Swoosh.Adapters.Mailjet.html#deliver_many/2"},{"type":"function","title":"Swoosh.Adapters.Mailjet.validate_config/1","doc":"","ref":"Swoosh.Adapters.Mailjet.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Mailjet.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Mailjet.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Mailtrap","doc":"An adapter that sends email using the Mailtrap API.\n\nFor reference: [Mailtrap API docs](https://api-docs.mailtrap.io/docs/mailtrap-api-docs/67f1d70aeb62c-send-email)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Mailtrap.html"},{"type":"module","title":"Example - Swoosh.Adapters.Mailtrap","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mailtrap,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Mailtrap.html#module-example"},{"type":"module","title":"Sandbox mode - Swoosh.Adapters.Mailtrap","doc":"For [sandbox mode](https://api-docs.mailtrap.io/docs/mailtrap-api-docs/bcf61cdc1547e-send-email-early-access), use the following config:\n\n    # config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mailtrap,\n      api_key: \"my-api-key\",\n      sandbox_inbox_id: \"111111\"","ref":"Swoosh.Adapters.Mailtrap.html#module-sandbox-mode"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Mailtrap","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Xu Shang-Chi\", \"xu.shangchi@example.com\"})\n    |> to({\"Katy\", \"katy@example.com\"})\n    |> reply_to(\"xu.xialing@example.com\")\n    |> cc(\"yingli@example.com\")\n    |> cc({\"Xu Wenwu\", \"xu.wenwu@example.com\"})\n    |> bcc(\"yingnan@example.com\")\n    |> bcc({\"Jon Jon\", \"jonjon@example.com\"})\n    |> subject(\"Hello, Ten Rings!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:custom_variables, %{\n      my_var: %{my_message_id: 123},\n      my_other_var: %{my_other_id: 1, stuff: 2}\n    })\n    |> put_provider_option(:category, \"welcome\")","ref":"Swoosh.Adapters.Mailtrap.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Mailtrap","doc":"Supported provider options are the following:\n\n#### Inserted into request body\n\n  * `:category` (string) - an email category\n\n  * `:custom_variables` (map) - a map contains fields","ref":"Swoosh.Adapters.Mailtrap.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Mailtrap.deliver/2","doc":"","ref":"Swoosh.Adapters.Mailtrap.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Mailtrap.validate_config/1","doc":"","ref":"Swoosh.Adapters.Mailtrap.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Mailtrap.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Mailtrap.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Mandrill","doc":"An adapter that sends email using the Mandrill API.\n\nIt supports both the `send` and `send-template` endpoint. In order to use the\nlatter you need to set `template_name` in the `provider_options` map on\n`Swoosh.Email`.\n\nFor reference: [Mandrill API docs](https://mandrillapp.com/api/docs/messages.html)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Mandrill.html"},{"type":"module","title":"Example - Swoosh.Adapters.Mandrill","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mandrill,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Mandrill.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Mandrill","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Rachel Chu\", \"rachel.chu@example.com\"})\n    |> to({\"Nick Young\", \"nick.young@example.com\"})\n    |> to(\"astrid.leongteo@example.com\")\n    |> reply_to(\"sk.starlight@example.com\")\n    |> cc({\"Goh Peik Lin\", \"goh.peiklin@example.com\"})\n    |> cc(\"goh.wyemun@example.com\")\n    |> bcc({\"Eleanor Sung-Young\", \"eleanor.sungyoung@example.com\"})\n    |> bcc(\"shang.suyi@example.com\")\n    |> subject(\"Hello, People!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:global_merge_vars, [\n      %{\"name\" => \"a\", \"content\" => \"b\"},\n      %{\"name\" => \"c\", \"content\" => \"d\"}\n    ])\n    |> put_provider_option(:merge_vars, [\n      %{\"rcpt\" => \"a@example.com\", \"vars\" => %{\"name\" => \"a\", \"content\" => \"b\"}},\n      %{\"rcpt\" => \"b@example.com\", \"vars\" => %{\"name\" => \"b\", \"content\" => \"b\"}},\n    ])\n    |> put_provider_option(:merge_language, \"mailchimp\"),\n    |> put_provider_option(:metadata, %{\"website\" => \"www.example.com\"})\n    |> put_provider_option(:template_name, \"welcome-user\")\n    |> put_provider_option(:template_content, [%{\"name\" => \"a\", \"content\" => \"b\"}])\n    |> put_provider_option(:subaccount, \"subaccount-x\")\n    |> put_provider_option(:tags, [\"tag-1\", \"tag-2\"])","ref":"Swoosh.Adapters.Mandrill.html#module-using-with-provider-options"},{"type":"module","title":"Provider options - Swoosh.Adapters.Mandrill","doc":"* `:global_merge_vars` (list[map]) - a list of maps of `:name` and\n    `:content` global variables for all recipients\n\n  * `:merge_language` (string) - merge tag language to use when evaluating\n    merge tags, and possible values are `mailchimp` or `handlebars`\n\n  * `:merge_vars` (list[map]) - a list of maps of `:rcpt` and `vars` for each\n    recipient, which will override `:global_merge_vars`\n\n  * `:metadata` (map) - a map of up to 10 fields for a user metadata\n\n  * `:template_content` (list[map]) - a list of maps of `:name` and\n    `:content` to be sent within a template\n\n  * `:template_name` (string) - a name of slug of the template belongs to a\n    user\n\n  * `:subaccount` (string) - the unique id of a subaccount for this message\n\n  * `:tags` (list[string]) - a list of strings to tag the message with","ref":"Swoosh.Adapters.Mandrill.html#module-provider-options"},{"type":"module","title":"Template-configured 'from' address - Swoosh.Adapters.Mandrill","doc":"Mandrill templates allow you to configure the 'from' address in the template itself.\nTo use the 'from' fields configured in the template, rather than specifying the value\nexplicitly, you can set\n\n  |> from(\"TEMPLATE\")","ref":"Swoosh.Adapters.Mandrill.html#module-template-configured-from-address"},{"type":"function","title":"Swoosh.Adapters.Mandrill.deliver/2","doc":"","ref":"Swoosh.Adapters.Mandrill.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Mandrill.validate_config/1","doc":"","ref":"Swoosh.Adapters.Mandrill.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Mandrill.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Mandrill.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.MsGraph","doc":"An adapter that sends email using the Microsoft Graph API.\n\nFor reference: [Microsoft Graph API docs](https://learn.microsoft.com/en-us/graph/api/user-sendmail)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.\n\n>","ref":"Swoosh.Adapters.MsGraph.html"},{"type":"module","title":"Dependency {: .info} - Swoosh.Adapters.MsGraph","doc":">\n> Microsoft Graph adapter requires `:gen_smtp` to work properly.\n> `:gen_smtp` is only used to encode the email body to MIME format.","ref":"Swoosh.Adapters.MsGraph.html#module-dependency-info"},{"type":"module","title":"Configuration options - Swoosh.Adapters.MsGraph","doc":"* `:auth` - either a function, a {mod, func, args} tuple, or a string that returns/is an OAuth 2.0 access token.\n* `:base_url` - the base URL to use as the Microsoft Graph API endpoint.  Defaults to the standard Microsoft Graph API endpoint.","ref":"Swoosh.Adapters.MsGraph.html#module-configuration-options"},{"type":"module","title":"Example - Swoosh.Adapters.MsGraph","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.MsGraph,\n      auth: fn -> Sample.OAuthTokenRequester.request_token() end\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.MsGraph.html#module-example"},{"type":"function","title":"Swoosh.Adapters.MsGraph.deliver/2","doc":"","ref":"Swoosh.Adapters.MsGraph.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.MsGraph.validate_config/1","doc":"","ref":"Swoosh.Adapters.MsGraph.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.MsGraph.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.MsGraph.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Mua","doc":"An adapter that sends email using the SMTP protocol.\n\n>","ref":"Swoosh.Adapters.Mua.html"},{"type":"module","title":"Dependency {: .info} - Swoosh.Adapters.Mua","doc":">\n> Underneath this adapter uses [Mua,](https://github.com/ruslandoga/mua) and\n> [Mail](https://github.com/DockYard/elixir-mail), and\n> [Castore](https://github.com/elixir-mint/castore) libraries,\n> add them to your mix.exs file.","ref":"Swoosh.Adapters.Mua.html#module-dependency-info"},{"type":"module","title":"Example - Swoosh.Adapters.Mua","doc":"# mix.exs\n    def deps do\n      [\n       {:swoosh, \"~> 1.3\"},\n       {:mua, \"~> 0.1.0\"},\n       {:mail, \"~> 0.3.0\"},\n       {:castore, \"~> 1.0\"}\n      ]\n    end\n\n    # config/config.exs for sending email directly\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mua\n\n    # config/config.exs for sending email via a relay\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mua,\n      relay: \"smtp.matrix.com\",\n      port: 1025,\n      auth: [username: \"neo\", password: \"one\"]\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end\n\nFor supported configuration options, please see [`option()`](#t:option/0)","ref":"Swoosh.Adapters.Mua.html#module-example"},{"type":"module","title":"Sending email directly - Swoosh.Adapters.Mua","doc":"When `relay` option is omitted, this adapter will send email directly to\nthe receivers' host. All receivers must be on the same host, otherwise\n`Swoosh.Adapters.Mua.MultihostError` is raised.\n\nIn this configuration, you need to ensure that your application can make\noutgoing connections to port 25 and that your sender domain has appropriate\nDNS records set, e.g. SPF or DKIM.\n\n> #### Short-lived connections {: .warning}\n>\n> Note that each `deliver` call results in a new connection to the receiver's\n> email server.","ref":"Swoosh.Adapters.Mua.html#module-sending-email-directly"},{"type":"module","title":"Sending email via a relay - Swoosh.Adapters.Mua","doc":"When `relay` option is set, this adapter will send email through that relay.\nThe relay would usually require authentication. For example, you can use your own\nGMail account with an app password.\n\n> #### Short-lived connections {: .warning}\n>\n> Note that each `deliver` call results in a new connection to the relay.\n> This is less efficient than `gen_smtp` which reuses the long-lived connection.\n> Further versions of this adapter might fix this if it ends up being a big problem ;)","ref":"Swoosh.Adapters.Mua.html#module-sending-email-via-a-relay"},{"type":"module","title":"CA certificates - Swoosh.Adapters.Mua","doc":"By default [`CAStore.file_path/0`](https://hexdocs.pm/castore/CAStore.html#file_path/0) is used for\n[`:cacertfile`,](https://www.erlang.org/doc/man/ssl#type-client_cafile) but you can provide your\nown or use [the system ones](https://www.erlang.org/doc/man/public_key#cacerts_get-0) and supply them\nin as [`:cacerts`](https://www.erlang.org/doc/man/ssl#type-client_cacerts)\n\n    :ok = :public_key.cacerts_load()\n    [_ | _] = cacerts = :public_key.cacerts_get()\n\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Mua,\n      transport_opts: [\n        cacerts: cacerts\n      ]\n\nNote that when using `:cacertfile` option, the certificates are decoded on each new connection.\nTo cache the decoded certificates, set `:persistent_term` for `:mua` to true.\n\n    config :mua, persistent_term: true","ref":"Swoosh.Adapters.Mua.html#module-ca-certificates"},{"type":"function","title":"Swoosh.Adapters.Mua.deliver/2","doc":"","ref":"Swoosh.Adapters.Mua.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Mua.validate_config/1","doc":"","ref":"Swoosh.Adapters.Mua.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Mua.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Mua.html#validate_dependency/0"},{"type":"type","title":"Swoosh.Adapters.Mua.option/0","doc":"","ref":"Swoosh.Adapters.Mua.html#t:option/0"},{"type":"module","title":"Swoosh.Adapters.Postmark","doc":"An adapter that sends email using the Postmark API.\n\nFor reference: [Postmark API docs](http://developer.postmarkapp.com/developer-send-api.html)","ref":"Swoosh.Adapters.Postmark.html"},{"type":"module","title":"Example - Swoosh.Adapters.Postmark","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Postmark,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Postmark.html#module-example"},{"type":"module","title":"Example of sending emails using templates - Swoosh.Adapters.Postmark","doc":"This will use Postmark's `withTemplate` endpoint.\n\n    import Swoosh.Email\n\n    new()\n    |> from({\"T Stark\", \"tony.stark@example.com\"})\n    |> to({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    |> put_provider_option(:template_id, \"123456\")\n    |> put_provider_option(:template_model, %{name: \"Steve\", email: \"steve@avengers.com\"})\n\nYou can also use `template_alias` instead of `template_id`, if you use Postmark's\n[TemplateAlias](https://postmarkapp.com/developer/api/templates-api#email-with-template) feature.\n\nNote that you must include the `:template_model` provider option even if your template\nhas no variables to interpolate. In this case you can pass an empty map:\n\n    put_provider_option(email, :template_model, %{})\n\nWhen sending batch emails using `:deliver_many` do not mix emails using\ntemplates with non-template emails. The use of templates impacts the API\nendpoint used and so the batch email collection should be of the same format.","ref":"Swoosh.Adapters.Postmark.html#module-example-of-sending-emails-using-templates"},{"type":"module","title":"Example of sending emails with a tag - Swoosh.Adapters.Postmark","doc":"This will add a tag to the sent Postmark's email.\n\n    import Swoosh.Email\n\n    new()\n    |> from({\"T Stark\", \"tony.stark@example.com\"})\n    |> to({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    |> subject(\"Hello, Avengers!\")\n    |> put_provider_option(:tag, \"some tag\")","ref":"Swoosh.Adapters.Postmark.html#module-example-of-sending-emails-with-a-tag"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Postmark","doc":"* `:message_stream` (string) – `MessageStream`, configure the message stream\n    for the email\n\n  * `:metadata` (map) - `Metadata`, add metadata to an email\n\n  * `:tag` (string) - `Tag`, to categorize outgoing email\n\n  * `:template_id` (string) - `TemplateId`, the template used when sending\n    email and only required if `:template_alias` is not specified\n\n  * `:template_alias` (string), `TemplateAlias`, the alias of a template used\n    when sending email and only required if `:template_id` is not specified\n\n  * `:template_model` (map), `TemplateModel`, a map of key/value field to be\n    used in the `HtmlBody`, `TextBody`, and `Subject` field in the template,\n    required alongside `:template_id`/`:template_alias`\n\n  * `:track_opens` (boolean) - `TrackOpens`, specify if open tracking needs to be enabled for this email.\n\n  * `:track_links` (string) - `TrackOpens`, specify if link tracking needs to be enabled for this email.\n     Valid values are: `None`, `HtmlAndText`, `HtmlOnly`, `TextOnly`\n\n  * `:inline_css` (boolean) - `InlineCss`, specify if Postmark should apply the style blocks as inline\n    attributes to the rendered HTML content. Default is true.","ref":"Swoosh.Adapters.Postmark.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Postmark.deliver/2","doc":"","ref":"Swoosh.Adapters.Postmark.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Postmark.deliver_many/2","doc":"","ref":"Swoosh.Adapters.Postmark.html#deliver_many/2"},{"type":"function","title":"Swoosh.Adapters.Postmark.validate_config/1","doc":"","ref":"Swoosh.Adapters.Postmark.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Postmark.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Postmark.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.ProtonBridge","doc":"An adapter that sends email using the local Protonmail Bridge.\n\nThis is a very thin wrapper around the SMTP adapter.\n\nUnderneath this adapter uses the\n[gen_smtp](https://github.com/Vagabond/gen_smtp) library, add it to your mix.exs file.","ref":"Swoosh.Adapters.ProtonBridge.html"},{"type":"module","title":"Example - Swoosh.Adapters.ProtonBridge","doc":"# mix.exs\n    def deps do\n      [\n        {:swoosh, \"~> 1.3\"},\n        {:gen_smtp, \"~> 1.1\"}\n      ]\n    end\n\n    # config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.ProtonBridge,\n      username: \"tonystark\",\n      password: \"ilovepepperpotts\",\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.ProtonBridge.html#module-example"},{"type":"module","title":"SMTP - Swoosh.Adapters.ProtonBridge","doc":"You can send emails with Protonmail SMTP service via the following SMTP configs,\nusing `Swoosh.Adapters.SMTP` adapter.\n\n    [\n      relay: \"smtp.protonmail.ch\",\n      ssl: false,\n      tls: :always,\n      auth: :always,\n      port: 587,\n      retries: 1,\n      no_mx_lookups: false\n    ]\n\nThis bridge adapter provides a special set of configs that utilize the local Protonmail Bridge.","ref":"Swoosh.Adapters.ProtonBridge.html#module-smtp"},{"type":"function","title":"Swoosh.Adapters.ProtonBridge.deliver/2","doc":"","ref":"Swoosh.Adapters.ProtonBridge.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.ProtonBridge.validate_config/1","doc":"","ref":"Swoosh.Adapters.ProtonBridge.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.ProtonBridge.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.ProtonBridge.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.SMTP","doc":"An adapter that sends email using the SMTP protocol.\n\nUnderneath this adapter uses the\n[gen_smtp](https://github.com/gen-smtp/gen_smtp) library, add it to your mix.exs file.","ref":"Swoosh.Adapters.SMTP.html"},{"type":"module","title":"Example - Swoosh.Adapters.SMTP","doc":"# mix.exs\n    def deps do\n      [\n       {:swoosh, \"~> 1.3\"},\n       {:gen_smtp, \"~> 1.1\"}\n      ]\n    end\n\n    # config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.SMTP,\n      relay: \"smtp.avengers.com\",\n      username: \"tonystark\",\n      password: \"ilovepepperpotts\",\n      ssl: true,\n      tls: :always,\n      auth: :always,\n      port: 1025,\n      dkim: [\n        s: \"default\", d: \"domain.com\",\n        private_key: {:pem_plain, File.read!(\"priv/keys/domain.private\")}\n      ],\n      retries: 2,\n      no_mx_lookups: false\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.SMTP.html#module-example"},{"type":"module","title":"Note - Swoosh.Adapters.SMTP","doc":"With `STARTTLS` you should omit the ssl configuration or set it to false.\n\nFor more details, please see [gen_smtp docs](https://hexdocs.pm/gen_smtp/readme.html)","ref":"Swoosh.Adapters.SMTP.html#module-note"},{"type":"function","title":"Swoosh.Adapters.SMTP.deliver/2","doc":"","ref":"Swoosh.Adapters.SMTP.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.SMTP.gen_smtp_config/1","doc":"","ref":"Swoosh.Adapters.SMTP.html#gen_smtp_config/1"},{"type":"function","title":"Swoosh.Adapters.SMTP.validate_config/1","doc":"","ref":"Swoosh.Adapters.SMTP.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.SMTP.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.SMTP.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.SMTP2GO","doc":"An adapter that sends email using the SMTP2GO API.\n\nFor reference: [SMTP2GO API docs](https://apidoc.smtp2go.com/documentation/#/POST%20/email/send)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.SMTP2GO.html"},{"type":"module","title":"Example - Swoosh.Adapters.SMTP2GO","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.SMTP2GO,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.SMTP2GO.html#module-example"},{"type":"module","title":"Usage - Swoosh.Adapters.SMTP2GO","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Tony\", \"ironman@example.com\"})\n    |> to({\"Thanos\", \"thanos@example.com\"})\n    |> reply_to(\"avengers@example.com\")\n    |> cc(\"hulk@example.com\")\n    |> bcc({\"Steve Rogers\", \"steve.rogers@example.com\"})\n    |> subject(\"I'm Ironman\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n\nwith template:\n\n    import Swoosh.Email\n\n    new()\n    |> from({\"Tony\", \"ironman@example.com\"})\n    |> to({\"Thanos\", \"thanos@example.com\"})\n    |> subject(\"I'm Ironman\")\n    |> put_provider_option(:template_id, \"123456\")\n    |> put_provider_option(:template_data, %{\"var1\" => \"value1\"})","ref":"Swoosh.Adapters.SMTP2GO.html#module-usage"},{"type":"function","title":"Swoosh.Adapters.SMTP2GO.deliver/2","doc":"","ref":"Swoosh.Adapters.SMTP2GO.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.SMTP2GO.validate_config/1","doc":"","ref":"Swoosh.Adapters.SMTP2GO.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.SMTP2GO.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.SMTP2GO.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Scaleway","doc":"An adapter that sends email using the Scaleway API (Transactional emails only).\n\nFor reference: [Scaleway API docs](https://www.scaleway.com/en/developers/api/transactional-email/#path-emails-send-an-email)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Scaleway.html"},{"type":"module","title":"Example - Swoosh.Adapters.Scaleway","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Scaleway,\n      project_id: \"my-project-id\",\n      secret_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Scaleway.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Scaleway","doc":"import Swoosh.Email\n\n    new()\n    |> from(\"nora@example.com\")\n    |> to(\"shushu@example.com\")\n    |> subject(\"Hello, Wally!\")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:send_before, ~U[2022-11-15 11:00:00Z])","ref":"Swoosh.Adapters.Scaleway.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Scaleway","doc":"* `send_before` (RFC 3339 format) - `send_before`, maximum date to deliver the email.","ref":"Swoosh.Adapters.Scaleway.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Scaleway.deliver/2","doc":"","ref":"Swoosh.Adapters.Scaleway.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Scaleway.validate_config/1","doc":"","ref":"Swoosh.Adapters.Scaleway.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Scaleway.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Scaleway.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Sendgrid","doc":"An adapter that sends email using the Sendgrid API.\n\nFor reference: [Sendgrid API docs](https://sendgrid.com/docs/API_Reference/Web_API_v3/Mail/index.html)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Sendgrid.html"},{"type":"module","title":"Example - Swoosh.Adapters.Sendgrid","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Sendgrid,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Sendgrid.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Sendgrid","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Xu Shang-Chi\", \"xu.shangchi@example.com\"})\n    |> to({\"Katy\", \"katy@example.com\"})\n    |> reply_to(\"xu.xialing@example.com\")\n    |> cc(\"yingli@example.com\")\n    |> cc({\"Xu Wenwu\", \"xu.wenwu@example.com\"})\n    |> bcc(\"yingnan@example.com\")\n    |> bcc({\"Jon Jon\", \"jonjon@example.com\"})\n    |> subject(\"Hello, Ten Rings!\")\n    |> html_body(\" Hello \")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:custom_args, %{\n      my_var: %{my_message_id: 123},\n      my_other_var: %{my_other_id: 1, stuff: 2}\n    })\n    |> put_provider_option(:asm, %{\n      \"group_id\" => 1,\n      \"groups_to_display\" => [1, 2, 3]\n    })\n    |> put_provider_option(:categories, [\"welcome\"])\n    |> put_provider_option(:mail_settings, %{\n      sandbox_mode: %{enable: true}\n    })\n    |> put_provider_option(:tracking_settings, %{\n      subscription_tracking: %{enable: false}\n    })\n    |> put_provider_option(:batch_id, \"AsdFgHjklQweRTYuIopzXcVBNm0aSDfGHjklmZcVbNMqWert1znmOP2asDFjkl\")\n    |> put_provider_option(:send_at, 1617260400)","ref":"Swoosh.Adapters.Sendgrid.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Sendgrid","doc":"Supported provider options are the following:\n\n#### Inserted into personalization\n\n  * `:custom_args` (map) - key/value pairs custom arguments that specific to\n    this personalization\n\n  * `:substitutions` (map) - key/value pairs of substitutions string applied\n    to the `:subject` and `:reply-to` parameter\n\n  * `:dynamic_template_data` (map) - key/value pairs of dynamic template data\n    used in Dynamic Transactional Templates, see `:template_id`\n\n#### Inserted into request body\n\n  * `:template_id` (string) - an email template ID\n\n  * `:asm` (map) - a map contains fields below on how to handle unsubscribes\n\n  * `:categories` (list[string]) - list of category name for this message\n\n  * `:mail_settings` (map) - collection of mail settings to handle this email\n\n  * `:tracking_settings` (map) - collection of settings to track the metrics\n    of responses of email recipients\n\n  * `:send_at` (integer) - A unix timestamp allowing you to specify when\n    you want your email to be delivered.\n\n  * `:batch_id` (string) - An ID representing a batch of emails to be sent at\n    the same time. It also enables you to cancel or pause the delivery of that batch","ref":"Swoosh.Adapters.Sendgrid.html#module-provider-options"},{"type":"module","title":"Sandbox mode - Swoosh.Adapters.Sendgrid","doc":"For [sandbox mode](https://sendgrid.com/docs/for-developers/sending-email/sandbox-mode/), use `put_provider_option/3`:\n\n    iex> new() |> put_provider_option(:mail_settings, %{sandbox_mode: %{enable: true}})","ref":"Swoosh.Adapters.Sendgrid.html#module-sandbox-mode"},{"type":"function","title":"Swoosh.Adapters.Sendgrid.deliver/2","doc":"","ref":"Swoosh.Adapters.Sendgrid.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Sendgrid.validate_config/1","doc":"","ref":"Swoosh.Adapters.Sendgrid.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Sendgrid.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Sendgrid.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Sendmail","doc":"An adapter that sends email using the sendmail binary.","ref":"Swoosh.Adapters.Sendmail.html"},{"type":"module","title":"Example - Swoosh.Adapters.Sendmail","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Sendmail,\n      cmd_path: \"/usr/bin/sendmail\",\n      cmd_args: \"-N delay,failure,success\",\n      qmail: true # Default false\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Sendmail.html#module-example"},{"type":"function","title":"Swoosh.Adapters.Sendmail.deliver/2","doc":"","ref":"Swoosh.Adapters.Sendmail.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Sendmail.validate_config/1","doc":"","ref":"Swoosh.Adapters.Sendmail.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Sendmail.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Sendmail.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.SocketLabs","doc":"An adapter that sends email using the SocketLabs Injection API.\n\nFor reference: [SocketLabs API docs](https://inject.docs.socketlabs.com/v1/documentation/introduction)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.SocketLabs.html"},{"type":"module","title":"Example - Swoosh.Adapters.SocketLabs","doc":"# config/config.exs\n    config :sample, Sample.Mailer\n      adapter: Swoosh.Adapters.SocketLabs,\n      server_id: \"\",\n      api_key: \"\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.SocketLabs.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.SocketLabs","doc":"import Swoosh.Email\n\n    new()\n    |> from({\"Sisu\", \"sisu@example.com\"})\n    |> to(\"raya@example.com\")\n    |> put_provider_option(:api_template, \"12345\")\n    |> put_provider_option(:charset, \"12345\")\n    |> put_provider_option(:mailing_id, \"12345\")\n    |> put_provider_option(:message_id, \"12345\")\n    |> put_provider_option(:merge_data, %{\n      \"PerMessage\" => %{\n        \"per_message1\" => \"value1\",\n        \"per_message2\" => \"value2\"\n      },\n      \"Global\" => %{\n        \"global1\" => \"value1\",\n        \"global2\" => \"value2\"\n      }\n    })","ref":"Swoosh.Adapters.SocketLabs.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.SocketLabs","doc":"* `:api_template` (string) - `ApiTemplate`, identifier for a content in the\n    Email Content Manager\n\n  * `:charset` (string) - `Charset`, character set used when creating the\n    email message and default to `UTF8`\n\n  * `:mailing_id` (string) - special header used to track batches of email\n    messages\n\n  * `:message_id` (string) - special header used to track individual message\n\n  * `:merge_data` (map) - data storage for inline Merge feature","ref":"Swoosh.Adapters.SocketLabs.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.SocketLabs.deliver/2","doc":"","ref":"Swoosh.Adapters.SocketLabs.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.SocketLabs.validate_config/1","doc":"","ref":"Swoosh.Adapters.SocketLabs.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.SocketLabs.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.SocketLabs.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.SparkPost","doc":"An adapter that sends email using the SparkPost API.\n\nFor reference: [SparkPost API docs](https://developers.sparkpost.com/api/)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.SparkPost.html"},{"type":"module","title":"Example - Swoosh.Adapters.SparkPost","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.SparkPost,\n      api_key: \"my-api-key\",\n      endpoint: \"https://api.sparkpost.com/api/v1\"\n      # or \"https://YOUR_DOMAIN.sparkpostelite.com/api/v1\" for enterprise\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.SparkPost.html#module-example"},{"type":"module","title":"Using with SparkPost templates - Swoosh.Adapters.SparkPost","doc":"import Swoosh.Email\n\n    new()\n    |> from(\"tony.stark@example.com\")\n    |> to(\"steve.rogers@example.com\")\n    |> subject(\"Hello, Avengers!\")\n    |> put_provider_option(:template_id, \"my-first-email\")\n    |> put_provider_option(:substitution_data, %{\n      first_name: \"Peter\",\n      last_name: \"Parker\"\n    })","ref":"Swoosh.Adapters.SparkPost.html#module-using-with-sparkpost-templates"},{"type":"module","title":"Setting SparkPost transmission options - Swoosh.Adapters.SparkPost","doc":"Full options can be found at [SparkPost Transmissions API Docs](https://developers.sparkpost.com/api/transmissions/#header-request-body)\n\n    import Swoosh.Email\n\n    new()\n    |> from(\"tony.stark@example.com\")\n    |> to(\"steve.rogers@example.com\")\n    |> subject(\"Hello, Avengers!\")\n    |> put_provider_option(:options, %{\n      click_tracking: false,\n      open_tracking: false,\n      transactional: true,\n      inline_css: true\n    })","ref":"Swoosh.Adapters.SparkPost.html#module-setting-sparkpost-transmission-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.SparkPost","doc":"* `:options` (map) - customization on how the email is sent\n\n  * `:template_id` (string) - id of the template to use\n\n  * `:substitution_data` (map) - data passed to the template language in\n    the content, and take precedence over the other data like `:metadata`","ref":"Swoosh.Adapters.SparkPost.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.SparkPost.deliver/2","doc":"","ref":"Swoosh.Adapters.SparkPost.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.SparkPost.validate_config/1","doc":"","ref":"Swoosh.Adapters.SparkPost.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.SparkPost.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.SparkPost.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Test","doc":"An adapter that sends emails as messages to the current process.\n\nThis is meant to be used during tests and works with the assertions found in\nthe [Swoosh.TestAssertions](Swoosh.TestAssertions.html) module.","ref":"Swoosh.Adapters.Test.html"},{"type":"module","title":"Example - Swoosh.Adapters.Test","doc":"# config/test.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Test\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Test.html#module-example"},{"type":"function","title":"Swoosh.Adapters.Test.deliver/2","doc":"","ref":"Swoosh.Adapters.Test.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Test.deliver_many/2","doc":"","ref":"Swoosh.Adapters.Test.html#deliver_many/2"},{"type":"function","title":"Swoosh.Adapters.Test.validate_config/1","doc":"","ref":"Swoosh.Adapters.Test.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Test.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Test.html#validate_dependency/0"},{"type":"behaviour","title":"Swoosh.ApiClient","doc":"Specification for a Swoosh API client.\n\nIt can be set to your own client with:\n\n    config :swoosh, :api_client, MyAPIClient\n\nSwoosh comes with `Swoosh.ApiClient.Hackney`, `Swoosh.ApiClient.Finch`, and\n`Swoosh.ApiClient.Req`.","ref":"Swoosh.ApiClient.html"},{"type":"callback","title":"Swoosh.ApiClient.init/0","doc":"Callback to initializes the given api client.","ref":"Swoosh.ApiClient.html#c:init/0"},{"type":"function","title":"Swoosh.ApiClient.post/4","doc":"API used by adapters to post to a given URL with headers, body, and email.","ref":"Swoosh.ApiClient.html#post/4"},{"type":"callback","title":"Swoosh.ApiClient.post/4","doc":"Callback invoked when posting to a given URL.","ref":"Swoosh.ApiClient.html#c:post/4"},{"type":"type","title":"Swoosh.ApiClient.body/0","doc":"","ref":"Swoosh.ApiClient.html#t:body/0"},{"type":"type","title":"Swoosh.ApiClient.headers/0","doc":"","ref":"Swoosh.ApiClient.html#t:headers/0"},{"type":"type","title":"Swoosh.ApiClient.status/0","doc":"","ref":"Swoosh.ApiClient.html#t:status/0"},{"type":"type","title":"Swoosh.ApiClient.url/0","doc":"","ref":"Swoosh.ApiClient.html#t:url/0"},{"type":"module","title":"Swoosh.ApiClient.Finch","doc":"Finch-based ApiClient for Swoosh.\n\n    config :swoosh, :api_client, Swoosh.ApiClient.Finch\n\nIn order to use `Finch` API client, you must start `Finch` and provide a :name.\nOften in your supervision tree:\n\n    children = [\n      {Finch, name: Swoosh.Finch}\n    ]\n\nOr, in rare cases, dynamically:\n\n    Finch.start_link(name: Swoosh.Finch)\n\nIf a name different from `Swoosh.Finch` is used, or you want to use an existing Finch instance,\nyou can provide the name via the config.\n\n    config :swoosh,\n      api_client: Swoosh.ApiClient.Finch,\n      finch_name: My.Custom.Name","ref":"Swoosh.ApiClient.Finch.html"},{"type":"module","title":"Swoosh.ApiClient.Hackney","doc":"Built-in hackney-based ApiClient.","ref":"Swoosh.ApiClient.Hackney.html"},{"type":"module","title":"Swoosh.ApiClient.Req","doc":"Req-based ApiClient for Swoosh.\n\n    config :swoosh, :api_client, Swoosh.ApiClient.Req\n\nAny `client_options` that are set will be passed along to `Req.post` but the\nfollowing keys will be overwritten as they are set by Swoosh explicitly:\n  * `headers`\n  * `body`\n  * `decode_body` - set to false as Adapters expect the raw response","ref":"Swoosh.ApiClient.Req.html"},{"type":"module","title":"Plug.Swoosh.MailboxPreview","doc":"Plug that serves pages useful for previewing emails in development.","ref":"Plug.Swoosh.MailboxPreview.html"},{"type":"module","title":"Examples - Plug.Swoosh.MailboxPreview","doc":"# in a Phoenix router\n    defmodule Sample.Router do\n      scope \"/dev\" do\n        pipe_through [:browser]\n        forward \"/mailbox\", Plug.Swoosh.MailboxPreview\n      end\n    end","ref":"Plug.Swoosh.MailboxPreview.html#module-examples"},{"type":"function","title":"Plug.Swoosh.MailboxPreview.call/2","doc":"","ref":"Plug.Swoosh.MailboxPreview.html#call/2"},{"type":"function","title":"Plug.Swoosh.MailboxPreview.init/1","doc":"","ref":"Plug.Swoosh.MailboxPreview.html#init/1"},{"type":"module","title":"Swoosh.TestAssertions","doc":"This module contains a set of assertions functions that you can import in your\ntest cases.\n\nIt is meant to be used with the\n[Swoosh.Adapters.Test](Swoosh.Adapters.Test.html) module.\n\n**Note**: `Swoosh.TestAssertion` works for unit tests and basic integration tests.\nUnfortunately it's not going to work for feature/E2E tests.\nThe mechanism of `assert_email_sent` is based on messaging sending between processes,\nand is expecting the calling process (the one that calls `assert_email_sent`) to be\nthe calling process of `Mailer.deliver`, or be the parent process of the whatever\ndoes the `Mailer.deliver` call.\n\nFor feature/E2E tests, you should use `Swoosh.Adapters.Local` adapter.\nIn your test, instead of calling `assert_email_sent`, you should navigate to the\npreview url with your E2E tool (e.g. `wallaby`) and test that the email is in the inbox.","ref":"Swoosh.TestAssertions.html"},{"type":"function","title":"Swoosh.TestAssertions.assert_email_not_sent/1","doc":"Asserts `email` was not sent.\n\nPerforms exact matching of the email struct.","ref":"Swoosh.TestAssertions.html#assert_email_not_sent/1"},{"type":"function","title":"Swoosh.TestAssertions.assert_email_sent/0","doc":"Asserts any email was sent.","ref":"Swoosh.TestAssertions.html#assert_email_sent/0"},{"type":"function","title":"Swoosh.TestAssertions.assert_email_sent/1","doc":"Asserts `email` was sent.\n\nYou can pass a keyword list to match on specific params\nor an anonymous function that returns a boolean.","ref":"Swoosh.TestAssertions.html#assert_email_sent/1"},{"type":"function","title":"Examples - Swoosh.TestAssertions.assert_email_sent/1","doc":"iex> alias Swoosh.Email\n    iex> import Swoosh.TestAssertions\n\n    iex> email = Email.new(subject: \"Hello, Avengers!\")\n    iex> Swoosh.Adapters.Test.deliver(email, [])\n\n    # assert a specific email was sent\n    iex> assert_email_sent(email)\n\n    # assert an email with specific field(s) was sent\n    iex> assert_email_sent(subject: \"Hello, Avengers!\")\n\n    # assert an email that satisfies a condition\n    iex> assert_email_sent(fn email ->\n    ...>   assert length(email.to) == 2\n    ...> end)","ref":"Swoosh.TestAssertions.html#assert_email_sent/1-examples"},{"type":"function","title":"Swoosh.TestAssertions.assert_emails_sent/0","doc":"Asserts multiple emails were sent.\n\nYou can pass a list of maps to match on specific params per email","ref":"Swoosh.TestAssertions.html#assert_emails_sent/0"},{"type":"function","title":"Examples - Swoosh.TestAssertions.assert_emails_sent/0","doc":"iex> alias Swoosh.Email\n    iex> import Swoosh.TestAssertions\n\n    iex> emails = Enum.map(1..2, fn n -> Email.new(subject: \"Hello, Avengers #{n}!\") end)\n    iex> Swoosh.Adapters.Test.deliver_many(emails, [])\n\n    # assert a specific email was sent\n    iex> assert_emails_sent(emails)\n\n    # assert the list of emails with specific field(s) that were sent\n    iex> assert_email_sent([\n      %{subject: \"Hello, Avengers 1!\"},\n      %{subject: \"Hello, Avengers 2!\"},\n    ])","ref":"Swoosh.TestAssertions.html#assert_emails_sent/0-examples"},{"type":"function","title":"Swoosh.TestAssertions.assert_emails_sent/1","doc":"","ref":"Swoosh.TestAssertions.html#assert_emails_sent/1"},{"type":"function","title":"Swoosh.TestAssertions.assert_no_email_sent/0","doc":"Asserts no emails were sent.","ref":"Swoosh.TestAssertions.html#assert_no_email_sent/0"},{"type":"macro","title":"Swoosh.TestAssertions.refute_email_sent/0","doc":"Asserts no emails were sent.","ref":"Swoosh.TestAssertions.html#refute_email_sent/0"},{"type":"macro","title":"Swoosh.TestAssertions.refute_email_sent/1","doc":"Asserts email with `attributes` was not sent.\n\nPerforms pattern matching using the given pattern, equivalent to `pattern = email`.\n\nWhen a list of attributes is given, they will be converted to a pattern.\n\nIt converts list fields (`:to`, `:cc`, `:bcc`) to a single element list if a single value is\ngiven (`to: \"email@example.com\"` => `to: [\"email@example.com\"]`).\n\nAfter conversion, performs pattern matching using a map of email attributes, similar to\n`%{attributes...} = email`.","ref":"Swoosh.TestAssertions.html#refute_email_sent/1"},{"type":"function","title":"Swoosh.TestAssertions.set_swoosh_global/1","doc":"Sets Swoosh test adapter to global mode.\n\nIn global mode, emails are consumed by the current test process,\ndoesn't matter which process sent it.\n\nAn ExUnit case where tests use Swoosh in global mode cannot be `async: true`.","ref":"Swoosh.TestAssertions.html#set_swoosh_global/1"},{"type":"function","title":"Examples - Swoosh.TestAssertions.set_swoosh_global/1","doc":"defmodule MyTest do\n      use ExUnit.Case, async: false\n\n      import Swoosh.Email\n      import Swoosh.TestAssertions\n\n      setup :set_swoosh_global\n\n      test \"it sends email\" do\n        # ...\n        assert_email_sent(subject: \"Hi Avengers!\")\n      end\n    end","ref":"Swoosh.TestAssertions.html#set_swoosh_global/1-examples"},{"type":"type","title":"Swoosh.TestAssertions.email_assertion/0","doc":"","ref":"Swoosh.TestAssertions.html#t:email_assertion/0"},{"type":"module","title":"Swoosh.X.TestAssertions","doc":"Experimental New TestAssertions Module that may replace the old new in v2.\n\nThis module contains a set of assertions functions that you can import in your\ntest cases.\n\nIt is meant to be used with the\n[Swoosh.Adapters.Test](Swoosh.Adapters.Test.html) module.\n\n**Note**: `Swoosh.X.TestAssertions` works for unit tests and basic integration tests.\nUnfortunately it's not going to work for feature/E2E tests.\nThe mechanism of `assert_email_sent` is based on messaging sending between processes,\nand is expecting the calling process (the one that calls `assert_email_sent`) to be\nthe calling process of `Mailer.deliver`, or be the parent process of the whatever\ndoes the `Mailer.deliver` call.\n\nFor feature/E2E tests, you should use `Swoosh.Adapters.Local` adapter.\nIn your test, instead of calling `assert_email_sent`, you should navigate to the\npreview url with your E2E tool (e.g. `wallaby`) and test that the email is in the inbox.","ref":"Swoosh.X.TestAssertions.html"},{"type":"function","title":"Swoosh.X.TestAssertions.assert_email_sent/0","doc":"Asserts any email was sent.","ref":"Swoosh.X.TestAssertions.html#assert_email_sent/0"},{"type":"function","title":"Swoosh.X.TestAssertions.assert_email_sent/1","doc":"Asserts `email` was sent.\n\nYou can pass a keyword list to match on specific params\nor an anonymous function that returns a boolean.","ref":"Swoosh.X.TestAssertions.html#assert_email_sent/1"},{"type":"function","title":"Examples - Swoosh.X.TestAssertions.assert_email_sent/1","doc":"iex> alias Swoosh.Email\n    iex> import Swoosh.X.TestAssertions\n\n    iex> email = Email.new(subject: \"Hello, Avengers!\")\n    iex> Swoosh.Adapters.Test.deliver(email, [])\n\n    # assert a specific email was sent\n    iex> assert_email_sent(email)\n\n    # assert an email with specific field(s) was sent\n    iex> assert_email_sent(subject: \"Hello, Avengers!\")\n\n    # assert an email that satisfies a condition\n    iex> assert_email_sent(fn email ->\n    ...>   assert length(email.to) == 2\n    ...> end)","ref":"Swoosh.X.TestAssertions.html#assert_email_sent/1-examples"},{"type":"function","title":"Swoosh.X.TestAssertions.flush_emails/0","doc":"Removes and returns from mailbox all sent emails.","ref":"Swoosh.X.TestAssertions.html#flush_emails/0"},{"type":"function","title":"Swoosh.X.TestAssertions.refute_email_sent/0","doc":"Asserts no emails were sent.","ref":"Swoosh.X.TestAssertions.html#refute_email_sent/0"},{"type":"function","title":"Swoosh.X.TestAssertions.refute_email_sent/1","doc":"Asserts email with `attributes` was not sent.\n\nYou can pass a keyword list to match on specific params\nor an anonymous function that returns a boolean.","ref":"Swoosh.X.TestAssertions.html#refute_email_sent/1"},{"type":"function","title":"Swoosh.X.TestAssertions.set_swoosh_global/1","doc":"Sets Swoosh test adapter to global mode.\n\nIn global mode, emails are consumed by the current test process,\ndoesn't matter which process sent it.\n\nAn ExUnit case where tests use Swoosh in global mode cannot be `async: true`.","ref":"Swoosh.X.TestAssertions.html#set_swoosh_global/1"},{"type":"function","title":"Examples - Swoosh.X.TestAssertions.set_swoosh_global/1","doc":"defmodule MyTest do\n      use ExUnit.Case, async: false\n\n      import Swoosh.Email\n      import Swoosh.X.TestAssertions\n\n      setup :set_swoosh_global\n\n      test \"it sends email\" do\n        # ...\n        assert_email_sent(subject: \"Hi Avengers!\")\n      end\n    end","ref":"Swoosh.X.TestAssertions.html#set_swoosh_global/1-examples"},{"type":"module","title":"Swoosh.Adapters.OhMySmtp","doc":"**Deprecated - use MailPace now**\n\n> Moving from OhMySMTP to MailPace\n> https://docs.mailpace.com/guide/moving_from_ohmysmtp\n\nAn adapter that sends email using the OhMySMTP API.\n\nFor reference: [OhMySMTP API docs](https://docs.ohmysmtp.com/reference/overview)\n\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.OhMySmtp.html"},{"type":"module","title":"Example - Swoosh.Adapters.OhMySmtp","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.OhMySmtp,\n      api_key: \"my-api-key\"\n\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.OhMySmtp.html#module-example"},{"type":"function","title":"Swoosh.Adapters.OhMySmtp.deliver/2","doc":"","ref":"Swoosh.Adapters.OhMySmtp.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.OhMySmtp.validate_config/1","doc":"","ref":"Swoosh.Adapters.OhMySmtp.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.OhMySmtp.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.OhMySmtp.html#validate_dependency/0"},{"type":"module","title":"Swoosh.Adapters.Sendinblue","doc":"**Deprecated - use Brevo now**\n\n> Moving from Sendinblue to Brevo\n> https://www.brevo.com/blog/becoming-brevo/\n\nAn adapter that sends email using the Sendinblue API (Transactional emails only).\nFor reference: [Sendinblue API docs](https://developers.sendinblue.com/reference/sendtransacemail)\n**This adapter requires an API Client.** Swoosh comes with Hackney, Finch and Req out of the box.\nSee the [installation section](https://hexdocs.pm/swoosh/Swoosh.html#module-installation)\nfor details.","ref":"Swoosh.Adapters.Sendinblue.html"},{"type":"module","title":"Example - Swoosh.Adapters.Sendinblue","doc":"# config/config.exs\n    config :sample, Sample.Mailer,\n      adapter: Swoosh.Adapters.Sendinblue,\n      api_key: \"my-api-key\"\n    # lib/sample/mailer.ex\n    defmodule Sample.Mailer do\n      use Swoosh.Mailer, otp_app: :sample\n    end","ref":"Swoosh.Adapters.Sendinblue.html#module-example"},{"type":"module","title":"Using with provider options - Swoosh.Adapters.Sendinblue","doc":"import Swoosh.Email\n    new()\n    |> from(\"nora@example.com\")\n    |> to(\"shushu@example.com\")\n    |> subject(\"Hello, Wally!\")\n    |> text_body(\"Hello\")\n    |> put_provider_option(:id, 42)\n    |> put_provider_option(:template_id, 42)\n    |> put_provider_option(:params, %{param1: \"a\", param2: 123})\n    |> put_provider_option(:tags, [\"tag_1\", \"tag_2\"])\n    |> put_provider_option(:schedule_at, ~U[2022-11-15 11:00:00Z])","ref":"Swoosh.Adapters.Sendinblue.html#module-using-with-provider-options"},{"type":"module","title":"Provider Options - Swoosh.Adapters.Sendinblue","doc":"* `sender_id` (integer) - `sender`, the sender `id` where this library will\n    add email obtained from the `from/1`\n  * `template_id` (integer) - `templateId`, the Id of the `active`\n    transactional email template\n  * `params` (map) - `params`, a map of key/value attributes to customize the\n    template\n  * `tags` (list[string]) - `tags`, a list of tags for each email for easy\n    filtering\n  * `schedule_at` (UTC DateTime) - `schedule_at`, a UTC date-time on which the email has to schedule","ref":"Swoosh.Adapters.Sendinblue.html#module-provider-options"},{"type":"function","title":"Swoosh.Adapters.Sendinblue.deliver/2","doc":"","ref":"Swoosh.Adapters.Sendinblue.html#deliver/2"},{"type":"function","title":"Swoosh.Adapters.Sendinblue.validate_config/1","doc":"","ref":"Swoosh.Adapters.Sendinblue.html#validate_config/1"},{"type":"function","title":"Swoosh.Adapters.Sendinblue.validate_dependency/0","doc":"","ref":"Swoosh.Adapters.Sendinblue.html#validate_dependency/0"},{"type":"exception","title":"Swoosh.Adapters.Mua.MultihostError","doc":"Raised when no relay is used and recipients contain addresses across multiple hosts.\n\nFor example:\n\n    email =\n      Swoosh.Email.new(\n        to: {\"Mua\", \"mua@github.com\"},\n        cc: [{\"Swoosh\", \"mua@swoosh.github.com\"}]\n      )\n\n    Swoosh.Adapters.Mua.deliver(email, _no_relay_config = [])\n\nFields:\n\n  - `:hosts` - the hosts for the recipients, `[\"github.com\", \"swoosh.github.com\"]` in the example above","ref":"Swoosh.Adapters.Mua.MultihostError.html"},{"type":"function","title":"Swoosh.Adapters.Mua.MultihostError.message/1","doc":"","ref":"Swoosh.Adapters.Mua.MultihostError.html#message/1"},{"type":"type","title":"Swoosh.Adapters.Mua.MultihostError.t/0","doc":"","ref":"Swoosh.Adapters.Mua.MultihostError.html#t:t/0"},{"type":"exception","title":"Swoosh.AttachmentContentError","doc":"","ref":"Swoosh.AttachmentContentError.html"},{"type":"exception","title":"Swoosh.DeliveryError","doc":"","ref":"Swoosh.DeliveryError.html"},{"type":"function","title":"Swoosh.DeliveryError.message/1","doc":"","ref":"Swoosh.DeliveryError.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.16.4 - Changelog","doc":"","ref":"changelog.html#1-16-4"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add support for allow\\_nil\\_from @bernardd (#895)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- remove unstable assertion @princemaple (#892)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.16.3 - Changelog","doc":"","ref":"changelog.html#1-16-3"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Mailbox Preview: more space for the HTML preview @justincy (#882)","ref":"changelog.html#features"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Fix Mailgun adapter incompatibility with Finch @AndrewDryga (#883)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.16.2 - Changelog","doc":"","ref":"changelog.html#1-16-2"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Update `MixProject` `xref` exclusions [`MultiPart.Part`] @jbcaprell (#880)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.16.1 - Changelog","doc":"With #877 Mailgun adapter now supports API Clients other than Hackney. Mailgun users,\nplease add [`:multipart`](https://hex.pm/packages/multipart) to your dependency list.","ref":"changelog.html#1-16-1"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Rewrite multipart functionality to use a encoding builder @krainboltgreene (#877)\n- Fix Req header handling @wojtekmach (#879)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.16.0 - Changelog","doc":"","ref":"changelog.html#1-16-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"Thank you @ruslandoga very much for throwing in this gem.\n\n- Add Swoosh.Adapters.Mua, an alternative SMTP adapter @ruslandoga (#870)","ref":"changelog.html#features"},{"type":"extras","title":"1.15.3 - Changelog","doc":"","ref":"changelog.html#1-15-3"},{"type":"extras","title":"✨ Features - Changelog","doc":"- [SMTP2GO] Pass more info down from the API response","ref":"changelog.html#features"},{"type":"extras","title":"1.15.2 - Changelog","doc":"","ref":"changelog.html#1-15-2"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add support of subaccount and tags for Mandrill @cenavarro (#860)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Clarify Postmark docs about template model @TheArrowsmith (#859)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.15.1 - Changelog","doc":"","ref":"changelog.html#1-15-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add support for Bandit @mtrudel (#857)","ref":"changelog.html#features"},{"type":"extras","title":"1.15.0 - Changelog","doc":"","ref":"changelog.html#1-15-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Support multiple reply\\_to in sendgrid @princemaple (#853)\n- Support reply\\_to in smtp2go @princemaple (#852)\n- Feat mailgun multiple reply to @ghostdsb (#850)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Improve docs on adapter functions and deliver_many in general","ref":"changelog.html#documentation"},{"type":"extras","title":"1.14.4 - Changelog","doc":"","ref":"changelog.html#1-14-4"},{"type":"extras","title":"✨ Features - Changelog","doc":"- AmazonSES: add :ses\\_source option to set Source API parameter @adamu (#846)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- fix comma issues on adapter config samples @SirWerto (#842)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.14.3 - Changelog","doc":"","ref":"changelog.html#1-14-3"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add template options @princemaple (#839)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- mention proton smtp, close #837 @princemaple (#840)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.14.2 - Changelog","doc":"","ref":"changelog.html#1-14-2"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Do not send subject to customer.io when empty @caioaao (#834)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Add information about Mailtrap adapter in README.md @kalys (#833)\n- Add req docs to Api Client section @krns (#831)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.14.1 - Changelog","doc":"","ref":"changelog.html#1-14-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add Swoosh.ApiClient.Req @matthewlehner (#830)","ref":"changelog.html#features"},{"type":"extras","title":"1.14.0 - Changelog","doc":"","ref":"changelog.html#1-14-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Implement Mailtrap adapter @kalys (#827)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Add a note about the Tailwindcss cdn when using a CSP @Hermanverschooten (#828)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.13.0 - Changelog","doc":"","ref":"changelog.html#1-13-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add Scaleway adapter @andreh11 (#825)\n- Update the UI for the mailbox viewer @dsincl12 (#822)","ref":"changelog.html#features"},{"type":"extras","title":"1.12.0 - Changelog","doc":"","ref":"changelog.html#1-12-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Implement Swoosh.Adapters.MsGraph @justindotpub (#815)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Update return value in docs @princemaple (#813)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.11.6 - Changelog","doc":"","ref":"changelog.html#1-11-6"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Add `plug` as an explicit dependency though still optional","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.11.5 - Changelog","doc":"","ref":"changelog.html#1-11-5"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Deprecate system env tuples @josevalim (#800)\n- Use concatenation to build sup children @josevalim (#801)\n- Compute docs lazily @josevalim (#802)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.11.4 - Changelog","doc":"","ref":"changelog.html#1-11-4"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Race condition on @on_load callback (#792) (quick fix in aef9cccbd)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Update Mailgun docs for sandbox mode @stjhimy (#787)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.11.3 - Changelog","doc":"","ref":"changelog.html#1-11-3"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Fix sections on CHANGELOG @nelsonmestevao (#781)","ref":"changelog.html#documentation"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- SendInBlue -> Brevo @princemaple (#783)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.11.2 - Changelog","doc":"","ref":"changelog.html#1-11-2"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Fix BCC for adapters that depend on SMTP helper @princemaple (#779)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Remove unnecessary sentence from README @adamu (#776)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.11.1 - Changelog","doc":"","ref":"changelog.html#1-11-1"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Do not include Bcc header in delivered email @adamu (#773) Thanks heaps for the discussion and PR!","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump mime from 2.0.3 to 2.0.5 @dependabot (#771)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.11.0 - Changelog","doc":"","ref":"changelog.html#1-11-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add experimental new test assertion module @jakub-gonet (#747)","ref":"changelog.html#features"},{"type":"extras","title":"1.10.3 - Changelog","doc":"","ref":"changelog.html#1-10-3"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"#### SMTP\n\n- Fix inline attachment showing up twice as both inline and attachment @Hermanverschooten (#769)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.10.2 - Changelog","doc":"","ref":"changelog.html#1-10-2"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Corrects typo in ex\\_aws\\_amazon\\_ses.ex @paynegreen (#766)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"1.10.1 - Changelog","doc":"","ref":"changelog.html#1-10-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Allow Regexp assertions for subjects @aronisstav (#764)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump finch from 0.15.0 to 0.16.0 @dependabot (#762)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.10.0 - Changelog","doc":"","ref":"changelog.html#1-10-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add assert\\_emails\\_sent @geeksilva97 (#757)\n- Add postmark inline\\_css option @matehat (#759)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Make Adapters.ExAwsAmazonSES easier to discover @nathanl (#749)\n- Add notes about API Client @Shadowbeetle (#743)","ref":"changelog.html#documentation"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Move docs above maintenance @princemaple (#760)\n- Bump ex\\_doc from 0.29.3 to 0.29.4 @dependabot (#756)\n- Bump ex\\_aws from 2.4.1 to 2.4.2 @dependabot (#755)\n- Bump ex\\_doc from 0.29.2 to 0.29.3 @dependabot (#753)\n- Bump plug\\_cowboy from 2.6.0 to 2.6.1 @dependabot (#752)\n- Bump finch from 0.14.0 to 0.15.0 @dependabot (#751)\n- Bump ex\\_doc from 0.29.1 to 0.29.2 @dependabot (#748)\n- Bump telemetry from 1.2.0 to 1.2.1 @dependabot (#745)\n- Bump telemetry from 1.1.0 to 1.2.0 @dependabot (#741)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.9.1 - Changelog","doc":"","ref":"changelog.html#1-9-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add support for Protonmail Bridge @Raphexion (#739)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Fix more typos @kianmeng (#736)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.9.0 - Changelog","doc":"","ref":"changelog.html#1-9-0"},{"type":"extras","title":"✨ New Adapter - Changelog","doc":"- Add customer.io adapter @lucacorti (#734)","ref":"changelog.html#new-adapter"},{"type":"extras","title":"1.8.3 - Changelog","doc":"**potential breaking change, fixing an unexpected behaviour**\n- Make return type of deliver\\_many consistent @princemaple (#733)","ref":"changelog.html#1-8-3"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Fix typo in contributor guidelines @nickcampbell18 (#727)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.8.2 - Changelog","doc":"","ref":"changelog.html#1-8-2"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Swoosh.Adapters.Test.delivery\\_many/2 returns a list @markthequark (#721)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Add missing double quote to mandrill template content sample @alvarezloaiciga (#726)","ref":"changelog.html#documentation"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump ex\\_doc from 0.28.5 to 0.29.0 @dependabot (#725)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.8.1 - Changelog","doc":"","ref":"changelog.html#1-8-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Postmark: Support per email tracking options @Wijnand (#722)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump jason from 1.3.0 to 1.4.0 @dependabot (#719)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.8.0 - Changelog","doc":"","ref":"changelog.html#1-8-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Prevent crashes caused by the memory GenServer restarts @KiKoS0 (#717)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump ex\\_aws from 2.3.4 to 2.4.0 @dependabot (#715)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.7.5 - Changelog","doc":"Bump to require Elixir 1.11. Now official support has been updated to Elixir 1.11+ with OTP 23+","ref":"changelog.html#1-7-5"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- doc: correct tags example for Adapters.Sendinblue @03juan (#711)","ref":"changelog.html#documentation"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump ex\\_doc from 0.28.4 to 0.28.5 @dependabot (#712)\n- Bump ex\\_aws from 2.3.3 to 2.3.4 @dependabot (#710)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.7.4 - Changelog","doc":"","ref":"changelog.html#1-7-4"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Set attachment's ContentId in Mailjet @marcinkoziej (#709)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump mime from 2.0.2 to 2.0.3 @dependabot (#708)\n- Bump finch from 0.12.0 to 0.13.0 @dependabot (#707)","ref":"changelog.html#maintenance"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Fix typos in gmail and socket labs adapters @zusoomro (#706)\n- Fix markdown issues and typos @kianmeng (#705)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.7.3 - Changelog","doc":"","ref":"changelog.html#1-7-3"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Support assertions for headers @MatheusBuss (#702)","ref":"changelog.html#features"},{"type":"extras","title":"1.7.2 - Changelog","doc":"","ref":"changelog.html#1-7-2"},{"type":"extras","title":"✨ Features - Changelog","doc":"- add schedule\\_at provider param for sendinblue @moperacz (#700)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Update Telemetry example to mention errors on `:stop` @lucasmazza (#698)","ref":"changelog.html#documentation"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump ex\\_aws from 2.3.2 to 2.3.3 @dependabot (#699)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.7.1 - Changelog","doc":"","ref":"changelog.html#1-7-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- sendgrid add support for scheduling emails @shravanjoopally (#696)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Test otp 25 @princemaple (#695)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.7.0 - Changelog","doc":"","ref":"changelog.html#1-7-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- SMTP: Allow send email without 'To' @Danielwsx64 (#694)\n- Add SMTP2GO adapter @princemaple (#687)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump ex\\_aws from 2.3.1 to 2.3.2 @dependabot (#692)\n- Bump finch from 0.11.0 to 0.12.0 @dependabot (#690)\n- Bump ex\\_doc from 0.28.3 to 0.28.4 @dependabot (#688)","ref":"changelog.html#maintenance"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- fix module name in ExAwsAmazonSES module doc @SteffenDE (#689)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.6.6 - Changelog","doc":"- Suppress warning about `ExAws.Config` introduced in 1.6.5 as optional dependency","ref":"changelog.html#1-6-6"},{"type":"extras","title":"1.6.5 - Changelog","doc":"- Add `Swoosh.Adapters.ExAwsAmazonSES` adapter @ascandella (#684)","ref":"changelog.html#1-6-5"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump gen_smtp from 1.1.1 to 1.2.0 @dependabot (#682)","ref":"changelog.html#maintenance"},{"type":"extras","title":"1.6.4 - Changelog","doc":"- Add message_stream documentation to Postmark adapter @ntodd (#674)\n- Rename Mime-Version header to MIME-Version @tcitworld (#681)","ref":"changelog.html#1-6-4"},{"type":"extras","title":"1.6.3 - Changelog","doc":"- Migrate OhMySmtp to Mailpace @princemaple (#672)","ref":"changelog.html#1-6-3"},{"type":"extras","title":"1.6.2 - Changelog","doc":"- SMTP can now utilize the new `:cid` addition in attachments, if `:cid` is\n  `nil` it will fallback to original behavior and use `:filename`\n- Fixed filename for inline images sent via SMTP","ref":"changelog.html#1-6-2"},{"type":"extras","title":"1.6.1 - Changelog","doc":"- Add fields to Postmark `deliver_many` response @zporter (#668)","ref":"changelog.html#1-6-1"},{"type":"extras","title":"1.6.0 - Changelog","doc":"","ref":"changelog.html#1-6-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- allow custom CIDs for inline attachments @taobojlen (#665)\n- add OhMySMTP adapter @taobojlen (#663)","ref":"changelog.html#features"},{"type":"extras","title":"🧰 Maintenance - Changelog","doc":"- Bump finch from 0.10.1 to 0.10.2 @dependabot (#661)\n- Bump ex_doc from 0.27.0 to 0.27.3 @dependabot (#660)\n- Bump ex_doc from 0.26.0 to 0.27.0 @dependabot (#659)\n- Bump finch from 0.10.0 to 0.10.1 @dependabot (#655)\n- Bump jason from 1.2.2 to 1.3.0 @dependabot (#654)\n- Bump finch from 0.9.1 to 0.10.0 @dependabot (#651)\n- Config bypass only on test @nallwhy (#650)","ref":"changelog.html#maintenance"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Mention E2E tests @princemaple (#664)\n- Add configuration options to Mailgun documentation @Zurga (#652)\n- Add example to Dyn adapter @kianmeng (#647)\n- Add provider options for Sparkpost @kianmeng (#646)\n- Add provider options doc for socketlabs @kianmeng (#645)\n- Update provider options doc for Sendinblue @kianmeng (#644)\n- Update provider options doc for Sendgrid @kianmeng (#643)\n- Update provider options doc for Postmark @kianmeng (#642)\n- Add provider options doc for Mandrill adapter @kianmeng (#641)\n- Add provider options doc for Mailjet @kianmeng (#640)\n- Update provider options doc for Mailgun adapter @kianmeng (#639)\n- Add provider options doc for Amazon SES adapter @kianmeng (#638)\n- Correct sample configuration for gmail adapter @aarongraham (#637)\n- Clarify that you need to add :gen_smtp as a dependency @Hermanverschooten (#635)","ref":"changelog.html#documentation"},{"type":"extras","title":"New Contributors - Changelog","doc":"- @Hermanverschooten made their first contribution in https://github.com/swoosh/swoosh/pull/635\n- @aarongraham made their first contribution in https://github.com/swoosh/swoosh/pull/637\n- @nallwhy made their first contribution in https://github.com/swoosh/swoosh/pull/650\n- @Zurga made their first contribution in https://github.com/swoosh/swoosh/pull/652\n- @taobojlen made their first contribution in https://github.com/swoosh/swoosh/pull/663\n\n**Full Changelog**: https://github.com/swoosh/swoosh/compare/v1.5.2...v1.6.0","ref":"changelog.html#new-contributors"},{"type":"extras","title":"1.5.2 - Changelog","doc":"","ref":"changelog.html#1-5-2"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix closing tag @feld (#634)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.5.1 - Changelog","doc":"","ref":"changelog.html#1-5-1"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Adding support for inline attachments preview in MailboxPreview @theodowling (#628)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Fixing Typo @Orijhins (#629)\n- Further cleanup async section @josevalim (#621)\n- Build upon async emails section @josevalim (#620)\n- Fix typos @kianmeng (#618)\n- Fix a few typos in the docs @nickjj (#617)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.5.0 - Changelog","doc":"","ref":"changelog.html#1-5-0"},{"type":"extras","title":"✨ Features - Changelog","doc":"- Add telemetry to `Mailer.deliver` \\& `Mailer.deliver_many` @joshnuss (#614)","ref":"changelog.html#features"},{"type":"extras","title":"📝 Documentation - Changelog","doc":"- Improve README.md - mention `api_client` as false @philss (#610)","ref":"changelog.html#documentation"},{"type":"extras","title":"1.4.0 - Changelog","doc":"","ref":"changelog.html#1-4-0"},{"type":"extras","title":"Add `Swoosh.ApiClient.Finch` - Changelog","doc":"You can configure what API Client to use by setting the config. Swoosh comes with\n`Swoosh.ApiClient.Hackney` and `Swoosh.ApiClient.Finch`\n\n```elixir\nconfig :swoosh, :api_client, MyAPIClient\n```\n\nIt defaults to use `:hackney` with `Swoosh.ApiClient.Hackney`. To use `Finch`,\nadd the below config\n\n```elixir\nconfig :swoosh, :api_client, Swoosh.ApiClient.Finch\n```\n\nTo use `Swoosh.ApiClient.Finch` you also need to start `Finch`, either in your\nsupervision tree\n\n```elixir\nchildren = [\n  {Finch, name: Swoosh.Finch}\n]\n```\n\nor somehow manually, and very rarely dynamically\n\n```elixir\nFinch.start_link(name: Swoosh.Finch)\n```\n\nIf a name different from `Swoosh.Finch` is used, or you want to use an existing\nFinch instance, you can provide the name via the config.\n\n```elixir\nconfig :swoosh,\n  api_client: Swoosh.ApiClient.Finch,\n  finch_name: My.Custom.Name\n```\n\n[Pre-1.4 changelogs](https://github.com/swoosh/swoosh/blob/v1.3.11/CHANGELOG.md)","ref":"changelog.html#add-swoosh-apiclient-finch"},{"type":"extras","title":"Contributor Code of Conduct","doc":"# Contributor Code of Conduct\n\nAs contributors and maintainers of this project, and in the interest of\nfostering an open and welcoming community, we pledge to respect all people who\ncontribute through reporting issues, posting feature requests, updating\ndocumentation, submitting pull requests or patches, and other activities.\n\nWe are committed to making participation in this project a harassment-free\nexperience for everyone, regardless of level of experience, gender, gender\nidentity and expression, sexual orientation, disability, personal appearance,\nbody size, race, ethnicity, age, religion, or nationality.\n\nExamples of unacceptable behavior by participants include:\n\n* The use of sexualized language or imagery\n* Personal attacks\n* Trolling or insulting/derogatory comments\n* Public or private harassment\n* Publishing other's private information, such as physical or electronic\n  addresses, without explicit permission\n* Other unethical or unprofessional conduct.\n\nProject maintainers have the right and responsibility to remove, edit, or\nreject comments, commits, code, wiki edits, issues, and other contributions\nthat are not aligned to this Code of Conduct. By adopting this Code of Conduct,\nproject maintainers commit themselves to fairly and consistently applying these\nprinciples to every aspect of managing this project. Project maintainers who do\nnot follow or enforce the Code of Conduct may be permanently removed from the\nproject team.\n\nThis code of conduct applies both within project spaces and in public spaces\nwhen an individual is representing the project or its community.\n\nInstances of abusive, harassing, or otherwise unacceptable behavior may be\nreported by opening an issue or contacting one or more of the project\nmaintainers.\n\nThis Code of Conduct is adapted from the\n[Contributor Covenant](http://contributor-covenant.org), version 1.2.0,\navailable at [http://contributor-covenant.org/version/1/2/0/](http://contributor-covenant.org/version/1/2/0/)","ref":"code_of_conduct.html"},{"type":"extras","title":"Contributing to Swoosh","doc":"# Contributing to Swoosh\n\nThanks for thinking about contributing to Swoosh. Please review this document\nfirst and also take a look at our [Code of Conduct](CODE_OF_CONDUCT.md), to\nhelp us keep this project inclusive to all those that may wish to contribute\nalso.","ref":"contributing.html"},{"type":"extras","title":"Opening Issues - Contributing to Swoosh","doc":"We classify bugs as any unexpected behaviour that occurs based on the code in\nthe project, and we really appreciate it when users take the time to\n[create an issue](https://github.com/swoosh/swoosh/issues).\n\nPlease take time to add as much detail as you can to any bug reports, consider\nincluding things like the version of Elixir you are using, which adapter you\nare having trouble with and any custom config you have passed for that adapter.\n\nIf you are thinking about adding a feature, you should the check Issues first,\nsomeone else may have started already, or it might be a feature we've decided\nnot to implement intentionally.","ref":"contributing.html#opening-issues"},{"type":"extras","title":"Submitting Pull Requests - Contributing to Swoosh","doc":"Whether you're fixing a bug, or proposing a feature you'd like to see included,\nyou can submit Pull Requests by following this guide:\n\n1. [Fork this repository](https://github.com/swoosh/swoosh/fork) and then clone\n   it locally:\n\n  ```bash\n  git clone https://github.com/swoosh/swoosh\n  ```\n\n2. Create a topic branch for your changes:\n\n  ```bash\n  git checkout -b fix-mailchimp-pricing-bug\n  ```\n\n3. Commit a failing test for the bug:\n\n  ```bash\n  git commit -am \"Adds a failing test that demonstrates the bug\"\n  ```\n\n4. Commit a fix that makes the test pass:\n\n  ```bash\n  git commit -am \"Adds a fix for the bug\"\n  ```\n\n5. Run the tests:\n\n  ```bash\n  mix test\n  ```\n\n6. If everything looks good, push to your fork:\n\n  ```bash\n  git push origin fix-mailchimp-pricing-bug\n  ```\n\n7. [Submit a pull request.](https://help.github.com/articles/creating-a-pull-request)","ref":"contributing.html#submitting-pull-requests"},{"type":"extras","title":"Style guidelines - Contributing to Swoosh","doc":"We support the common conventions found in Elixir, if you're in doubt take a\nlook at the code in the project, and we would like to keep the style consistent\nthroughout.","ref":"contributing.html#style-guidelines"},{"type":"extras","title":"General rules - Contributing to Swoosh","doc":"- Keep line length below 120 characters.\n- Complex anonymous functions should be extracted into named functions.\n- One line functions, should only take up one line!\n- Pipes are great, but don't use them, if they are less readable than brackets\n  then drop the pipe!","ref":"contributing.html#general-rules"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}